@@ -7,16 +7,23 @@ import re
 import uuid
 import logging
 import itertools
+from html import escape
 from collections import OrderedDict
 from os.path import abspath, dirname, join
 
 from jinja2 import Environment, FileSystemLoader
 
 from indra.statements import *
+from indra.sources import SOURCE_INFO
+from indra.statements.agent import default_ns_order
+from indra.statements.validate import validate_id
+from indra.databases.identifiers import get_identifiers_url, ensure_prefix
 from indra.assemblers.english import EnglishAssembler, AgentWithCoordinates
-from indra.databases import get_identifiers_url
 from indra.util.statement_presentation import group_and_sort_statements, \
-    make_top_level_label_from_names_key, make_stmt_from_sort_key
+    make_top_level_label_from_names_key, make_stmt_from_sort_key, \
+    reader_sources, db_sources, all_sources, get_available_source_counts, \
+    get_available_ev_counts, standardize_counts
+from indra.literature import id_lookup
 
 logger = logging.getLogger(__name__)
 HERE = dirname(abspath(__file__))
@@ -30,8 +37,9 @@ default_template = env.get_template('indra/statements_view.html')
 color_schemes = {
     'dark': ['#b2df8a', '#000099', '#6a3d9a', '#1f78b4', '#fdbf6f', '#ff7f00',
              '#cab2d6', '#fb9a99', '#a6cee3', '#33a02c', '#b15928', '#e31a1c'],
-    'light': ['#bc80bd', '#fccde5', '#b3de69', '#80b1d3', '#fb8072', '#bebada',
-              '#fdb462', '#8dd3c7', '#ffffb3', '#d9d9d9', '#ccebc5', '#ffed6f']
+    'light': ['#bebada', '#fdb462', '#b3de69', '#80b1d3', '#bc80bd', '#fccde5',
+              '#fb8072', '#d9d9d9', '#8dd3c7', '#ffed6f', '#ccebc5', '#e0e03d',
+              '#ffe8f4', '#acfcfc', '#dd99ff', '#00d4a6']
 }
 
 
@@ -41,21 +49,18 @@ def color_gen(scheme):
             yield color
 
 
-SOURCE_COLORS = [
-    ('databases', {'color': 'black',
-                   'sources': dict(zip(['phosphosite', 'cbn', 'pc11',
-                                        'biopax', 'bel_lc',
-                                        'signor', 'biogrid', 'tas',
-                                        'lincs_drug', 'hprd', 'trrust'],
-                                       color_gen('light')))}),
-    ('reading', {'color': 'white',
-                 'sources': dict(zip(['geneways', 'tees', 'isi', 'trips',
-                                      'rlimsp', 'medscan', 'sparser', 'reach'],
-                                     color_gen('light')))}),
-]
+def make_source_colors(databases, readers):
+    rdr_ord = ['reach', 'sparser', 'medscan', 'trips', 'eidos']
+    readers.sort(key=lambda r: rdr_ord.index(r) if r in rdr_ord else len(rdr_ord))
+    reader_colors_list = list(zip(readers, color_gen('light')))
+    reader_colors_list.reverse()
+    reader_colors = dict(reader_colors_list)
+    db_colors = dict(zip(databases, color_gen('light')))
+    return [('databases', {'color': 'black', 'sources': db_colors}),
+            ('reading', {'color': 'white', 'sources': reader_colors})]
 
-SRC_KEY_DICT = {src: src for _, d in SOURCE_COLORS
-                for src in d['sources'].keys()}
+
+DEFAULT_SOURCE_COLORS = make_source_colors(db_sources, reader_sources)
 
 
 class HtmlAssembler(object):
@@ -80,12 +85,18 @@ class HtmlAssembler(object):
         INDRA REST API. Default is None. Each value should be a concise
         summary of O(1), not of order the length of the list, such as the
         evidence totals. The keys should be informative human-readable strings.
-    ev_totals : Optional[dict]
+    ev_counts : Optional[dict]
         A dictionary of the total evidence available for each
-        statement indexed by hash. Default: None
+        statement indexed by hash. If not provided, the statements that are
+        passed to the constructor are used to determine these, with whatever
+        evidences these statements carry.
+    ev_totals : Optional[dict]
+        DEPRECATED. Same as ev_counts which should be used instead.
     source_counts : Optional[dict]
         A dictionary of the itemized evidence counts, by source, available for
-        each statement, indexed by hash. Default: None.
+        each statement, indexed by hash. If not provided, the statements
+        that are passed to the constructor are used to determine these, with
+        whatever evidences these statements carry.
     title : str
         The title to be printed at the top of the page.
     db_rest_url : Optional[str]
@@ -105,7 +116,7 @@ class HtmlAssembler(object):
     metadata : dict
         Dictionary of statement list metadata such as that provided by the
         INDRA REST API.
-    ev_totals : dict
+    ev_counts : dict
         A dictionary of the total evidence available for each
         statement indexed by hash.
     db_rest_url : str
@@ -113,14 +124,19 @@ class HtmlAssembler(object):
     """
 
     def __init__(self, statements=None, summary_metadata=None, ev_totals=None,
-                 source_counts=None, curation_dict=None, title='INDRA Results',
-                 db_rest_url=None):
+                 ev_counts=None, source_counts=None, curation_dict=None,
+                 title='INDRA Results', db_rest_url=None):
         self.title = title
         self.statements = [] if statements is None else statements
         self.metadata = {} if summary_metadata is None \
             else summary_metadata
-        self.ev_totals = {} if ev_totals is None else ev_totals
-        self.source_counts = {} if source_counts is None else source_counts
+        # If the deprecated parameter is used, we make sure we take it
+        if not ev_counts and ev_totals:
+            ev_counts = ev_totals
+        self.ev_counts = get_available_ev_counts(self.statements) \
+            if ev_counts is None else standardize_counts(ev_counts)
+        self.source_counts = get_available_source_counts(self.statements) \
+            if source_counts is None else standardize_counts(source_counts)
         self.curation_dict = {} if curation_dict is None else curation_dict
         self.db_rest_url = db_rest_url
         self.model = None
@@ -135,15 +151,21 @@ class HtmlAssembler(object):
         """
         self.statements += statements
 
-    def make_json_model(self, with_grouping=True):
+    def make_json_model(self, with_grouping=True, no_redundancy=False):
         """Return the JSON used to create the HTML display.
 
         Parameters
         ----------
-        with_grouping : bool
+        with_grouping : Optional[bool]
             If True, statements will be grouped under multiple sub-headings. If
             False, all headings will be collapsed into one on every level, with
-            all statements placed under a single heading.
+            all statements placed under a single heading. Default: False
+        no_redundancy : Optional[bool]
+            If True, any group of statements that was already presented under
+            a previous heading will be skipped. This is typically the case
+            for complexes where different permutations of complex members
+            are presented. By setting this argument to True, these can be
+            eliminated. Default: False
 
         Returns
         -------
@@ -154,13 +176,14 @@ class HtmlAssembler(object):
         # Get an iterator over the statements, carefully grouped.
         stmt_rows = group_and_sort_statements(
             self.statements,
-            self.ev_totals if self.ev_totals else None,
+            self.ev_counts if self.ev_counts else None,
             self.source_counts if self.source_counts else None)
 
         # Do some extra formatting.
         stmts = OrderedDict()
         agents = {}
         previous_stmt_set = set()
+        all_previous_stmts = set()
         for row in stmt_rows:
             # Distinguish between the cases with source counts and without.
             if self.source_counts:
@@ -172,8 +195,11 @@ class HtmlAssembler(object):
             curr_stmt_set = {s.get_hash() for s in stmts_group}
             if curr_stmt_set == previous_stmt_set:
                 continue
+            elif no_redundancy and curr_stmt_set <= all_previous_stmts:
+                continue
             else:
                 previous_stmt_set = curr_stmt_set
+                all_previous_stmts |= curr_stmt_set
 
             # We will keep track of some of the meta data for this stmt group.
             # NOTE: Much of the code relies heavily on the fact that the Agent
@@ -202,8 +228,8 @@ class HtmlAssembler(object):
                 # Format some strings nicely.
                 ev_list = _format_evidence_text(stmt, self.curation_dict)
                 english = _format_stmt_text(stmt)
-                if self.ev_totals:
-                    tot_ev = self.ev_totals.get(int(stmt_hash), '?')
+                if self.ev_counts:
+                    tot_ev = self.ev_counts.get(int(stmt_hash), '?')
                     if tot_ev == '?':
                         logger.warning('The hash %s was not found in the '
                                        'evidence totals dict.' % stmt_hash)
@@ -229,7 +255,12 @@ class HtmlAssembler(object):
                         del ag.db_refs[dbn]
 
             # Update the top level grouping.
-            tl_names = key[1]
+            if isinstance(stmt, (ActiveForm, HasActivity)):
+                tl_names = [key[1][0]]
+            elif isinstance(stmt, Conversion):
+                tl_names = [key[1][0]] + [*key[1][1]] + [*key[1][2]]
+            else:
+                tl_names = key[1]
             if with_grouping:
                 tl_key = '-'.join([str(name) for name in tl_names])
                 tl_agents = {name: Agent(name) for name in tl_names
@@ -299,7 +330,9 @@ class HtmlAssembler(object):
 
         return stmts
 
-    def make_model(self, template=None, with_grouping=True, **template_kwargs):
+    def make_model(self, template=None, with_grouping=True,
+                   add_full_text_search_link=False, no_redundancy=False,
+                   **template_kwargs):
         """Return the assembled HTML content as a string.
 
         Parameters
@@ -312,17 +345,44 @@ class HtmlAssembler(object):
             If True, statements will be grouped under multiple sub-headings. If
             False, all headings will be collapsed into one on every level, with
             all statements placed under a single heading.
+        add_full_text_search_link : bool
+            If True, link with Text fragment search in PMC journal will be
+            added for the statements.  
+        no_redundancy : Optional[bool]
+            If True, any group of statements that was already presented under
+            a previous heading will be skipped. This is typically the case
+            for complexes where different permutations of complex members
+            are presented. By setting this argument to True, these can be
+            eliminated. Default: False
 
-        All other keyword arguments are passed along to the template. If you
-        are using a custom template with args that are not passed below, this
-        is how you pass them.
+            All other keyword arguments are passed along to the template. If you
+            are using a custom template with args that are not passed below, this
+            is how you pass them.
 
         Returns
         -------
         str
             The assembled HTML as a string.
         """
-        tl_stmts = self.make_json_model(with_grouping)
+        tl_stmts = self.make_json_model(with_grouping,
+                                        no_redundancy=no_redundancy)
+
+        if add_full_text_search_link:
+            for statement in tl_stmts:
+                statement = tl_stmts[statement]
+                for stmt_formatted in statement["stmts_formatted"]:
+                    for stmt_info in stmt_formatted["stmt_info_list"]:
+                        for evidence in stmt_info["evidence"]:
+                            if 'PMCID' not in evidence.get('text_refs', {}):
+                                if evidence.get('pmid'):
+                                    ev_pmcid = id_lookup(
+                                        evidence['pmid'], 'pmid') \
+                                        .get('pmcid', None)
+                                    if ev_pmcid:
+                                        evidence['pmcid'] = ev_pmcid
+                            else:
+                                evidence['pmcid'] = \
+                                    evidence['text_refs']['PMCID']
 
         metadata = {k.replace('_', ' ').title(): v
                     for k, v in self.metadata.items()
@@ -336,13 +396,19 @@ class HtmlAssembler(object):
         if template is None:
             template = default_template
         if self.source_counts and 'source_key_dict' not in template_kwargs:
-            template_kwargs['source_key_dict'] = SRC_KEY_DICT
+            template_kwargs['source_key_dict'] = \
+                {src: src for src in all_sources}
         if 'source_colors' not in template_kwargs:
-            template_kwargs['source_colors'] = SOURCE_COLORS
+            template_kwargs['source_colors'] = DEFAULT_SOURCE_COLORS
+        if 'source_info' not in template_kwargs:
+            template_kwargs['source_info'] = SOURCE_INFO.copy()
+        if 'simple' not in template_kwargs:
+            template_kwargs['simple'] = True
 
         self.model = template.render(stmt_data=tl_stmts,
                                      metadata=metadata, title=self.title,
                                      db_rest_url=db_rest_url,
+                                     add_full_text_search_link=add_full_text_search_link,  # noqa
                                      **template_kwargs)
         return self.model
 
@@ -416,6 +482,7 @@ def _format_evidence_text(stmt, curation_dict=None, correct_tags=None):
         if ev.text is None:
             format_text = None
         else:
+            ev_text = escape(ev.text)
             indices = []
             for ix, ag in enumerate(stmt.agent_list()):
                 if ag is None:
@@ -431,6 +498,7 @@ def _format_evidence_text(stmt, curation_dict=None, correct_tags=None):
                     ag_text = ag.db_refs.get('TEXT')
                 if ag_text is None:
                     continue
+                ag_text = escape(ag_text)
                 role = get_role(ix)
                 # Get the tag with the correct badge
                 tag_start = '<span class="badge badge-%s">' % role
@@ -438,8 +506,8 @@ def _format_evidence_text(stmt, curation_dict=None, correct_tags=None):
                 # Build up a set of indices
                 indices += [(m.start(), m.start() + len(ag_text),
                              ag_text, tag_start, tag_close)
-                            for m in re.finditer(re.escape(ag_text), ev.text)]
-            format_text = tag_text(ev.text, indices)
+                            for m in re.finditer(re.escape(ag_text), ev_text)]
+            format_text = tag_text(ev_text, indices)
 
         curation_key = (stmt.get_hash(), ev.source_hash)
         curations = curation_dict.get(curation_key, [])
@@ -447,9 +515,10 @@ def _format_evidence_text(stmt, curation_dict=None, correct_tags=None):
         num_correct = len(
             [cur for cur in curations if cur['error_type'] in correct_tags])
         num_incorrect = num_curations - num_correct
+        text_refs = {k.upper(): v for k, v in ev.text_refs.items()}
         ev_list.append({'source_api': source_api,
                         'pmid': ev.pmid,
-                        'text_refs': ev.text_refs,
+                        'text_refs': text_refs,
                         'text': format_text,
                         'source_hash': str(ev.source_hash),
                         'num_curations': num_curations,
@@ -495,10 +564,11 @@ def tag_agents(english, agents):
             continue
         url = id_url(ag)
         if url is None:
-            continue
-        # Build up a set of indices
-        tag_start = "<a href='%s' target='_blank'>" % url
-        tag_close = "</a>"
+            tag_start = '<b>'
+            tag_close = '</b>'
+        else:
+            tag_start = "<a href='%s' target='_blank'>" % url
+            tag_close = "</a>"
         # If coordinates are passed, use them. Otherwise, try to find agent
         # names in english text
         if isinstance(ag, AgentWithCoordinates):
@@ -518,26 +588,35 @@ def tag_agents(english, agents):
     return tag_text(english, indices)
 
 
+link_namespace_order = default_ns_order + \
+    ['CHEMBL', 'DRUGBANK', 'PUBCHEM', 'HMDB', 'HMS-LINCS', 'CAS',
+     'IP', 'PF', 'NXPFA', 'MIRBASEM', 'NCIT', 'WM']
+
+
 def id_url(ag):
     # Return identifier URLs in a prioritized order
-    for db_name in ('FPLX', 'HGNC', 'UP',
-                    'GO', 'MESH',
-                    'CHEBI', 'PUBCHEM', 'HMDB',
-                    'IP', 'PF', 'NXPFA',
-                    'MIRBASEM', 'MIRBASE',
-                    'NCIT',
-                    'WM', 'UN', 'HUME', 'CWMS', 'SOFIA'):
+    # TODO: we should add handling for UPPRO here, however, that would require
+    # access to UniProt client resources in the context of the DB REST API
+    # which could be problematic
+    for db_name in link_namespace_order:
         if db_name in ag.db_refs:
             # Handle a special case where a list of IDs is given
             if isinstance(ag.db_refs[db_name], list):
                 db_id = ag.db_refs[db_name][0]
-                if db_name == 'CHEBI':
-                    if not db_id.startswith('CHEBI'):
-                        db_id = 'CHEBI:%s' % db_id
-                elif db_name in ('UN', 'WM', 'HUME'):
+                if db_name == 'WM':
                     db_id = db_id[0]
             else:
                 db_id = ag.db_refs[db_name]
+            # We can add more name spaces here if there are issues
+            if db_name in {'CHEBI'}:
+                db_id = ensure_prefix('CHEBI', db_id)
+            # Here we validate IDs to make sure we don't surface invalid
+            # links.
+            if not validate_id(db_name, db_id):
+                logger.debug('Invalid grounding encountered: %s:%s' %
+                             (db_name, db_id))
+                continue
+            # Finally, we return a valid identifiers.org URL
             return get_identifiers_url(db_name, db_id)
 
 
@@ -561,7 +640,6 @@ def tag_text(text, tag_info_list):
         String where the specified substrings have been surrounded by the
         given start and close tags.
     """
-
     # Check to tags for overlap and if there is any, return the subsumed
     # range. Return None if no overlap.
     def overlap(t1, t2):
