@@ -1,6 +1,5 @@
-from .model_checker import ModelChecker, PathResult, PathMetric, get_path_iter
+from .model_checker import ModelChecker, PathResult, PathMetric
 from .pysb import PysbModelChecker
 from .signed_graph import SignedGraphModelChecker
 from .unsigned_graph import UnsignedGraphModelChecker
 from .pybel import PybelModelChecker
-from .model_checker import signed_edges_to_signed_nodes, prune_signed_nodes
