@@ -1,12 +1,12 @@
 import logging
 import textwrap
-import itertools
-from collections import deque
 from copy import deepcopy
 
 import numpy as np
 import networkx as nx
 
+from indra.explanation.pathfinding import get_path_iter, find_sources
+
 try:
     import paths_graph as pg
     has_pg = True
@@ -142,13 +142,17 @@ class ModelChecker(object):
         generate paths. Default is False (breadth-first search).
     seed : int
         Random seed for sampling (optional, default is None).
+    nodes_to_agents : dict
+        A dictionary mapping nodes of intermediate signed edges graph to INDRA
+        agents.
 
     Attributes
     ----------
     graph : nx.Digraph
         A DiGraph with signed nodes to find paths in.
     """
-    def __init__(self, model, statements=None, do_sampling=False, seed=None):
+    def __init__(self, model, statements=None, do_sampling=False, seed=None,
+                 nodes_to_agents=None):
         self.model = model
         if statements:
             self.statements = statements
@@ -156,6 +160,7 @@ class ModelChecker(object):
             self.statements = []
         if seed is not None:
             np.random.seed(seed)
+        self.nodes_to_agents = nodes_to_agents if nodes_to_agents else {}
         # Whether to do sampling
         self.do_sampling = do_sampling
         self.graph = None
@@ -170,7 +175,8 @@ class ModelChecker(object):
         """
         self.statements += stmts
 
-    def check_model(self, max_paths=1, max_path_length=5):
+    def check_model(self, max_paths=1, max_path_length=5,
+                    agent_filter_func=None):
         """Check all the statements added to the ModelChecker.
 
         Parameters
@@ -180,6 +186,10 @@ class ModelChecker(object):
             to be explained. Default: 1
         max_path_length : Optional[int]
             The maximum length of specific paths to return. Default: 5
+        agent_filter_func : Optional[function]
+            A function to constrain the intermediate nodes in the path. A
+            function should take an agent as a parameter and return True if the
+            agent is allowed to be in a path and False otherwise.
 
         Returns
         -------
@@ -188,15 +198,19 @@ class ModelChecker(object):
             a PathResult object describing the results of model checking.
         """
         results = []
+        # Convert agent filter function to node filter function once here
+        node_filter_func = self.update_filter_func(agent_filter_func)
         for idx, stmt in enumerate(self.statements):
             logger.info('---')
             logger.info('Checking statement (%d/%d): %s' %
                         (idx + 1, len(self.statements), stmt))
-            result = self.check_statement(stmt, max_paths, max_path_length)
+            result = self.check_statement(stmt, max_paths, max_path_length,
+                                          node_filter_func=node_filter_func)
             results.append((stmt, result))
         return results
 
-    def check_statement(self, stmt, max_paths=1, max_path_length=5):
+    def check_statement(self, stmt, max_paths=1, max_path_length=5,
+                        agent_filter_func=None, node_filter_func=None):
         """Check a single Statement against the model.
 
         Parameters
@@ -208,19 +222,72 @@ class ModelChecker(object):
             to be explained. Default: 1
         max_path_length : Optional[int]
             The maximum length of specific paths to return. Default: 5
+        agent_filter_func : Optional[function]
+            A function to constrain the intermediate nodes in the path. A
+            function should take an agent as a parameter and return True if the
+            agent is allowed to be in a path and False otherwise.
+        node_filter_func : Optional[function]
+            Similar to agent_filter_func but it takes a node as a parameter
+            instead of agent. If not provided, node_filter_func will be
+            generated from agent_filter_func.
 
         Returns
         -------
         result : indra.explanation.modelchecker.PathResult
             A PathResult object containing the result of a test.
         """
+        input_set, obj_list, result_code = self.get_all_subjects_objects(stmt)
+        if result_code:
+            return self.make_false_result(result_code, max_paths,
+                                          max_path_length)
+        # If source and target are the same, we need to handle a loop
+        loop = False
+        if (input_set and (len(input_set) == len(obj_list) == 1) and
+                (list(input_set)[0] == list(obj_list)[0])):
+            loop = True
+
+        # Convert agent filter function to node filter function
+        if agent_filter_func and not node_filter_func:
+            node_filter_func = self.update_filter_func(agent_filter_func)
+        # If we have several objects in obj_list or we have a loop, we add a
+        # dummy target node as a child to all nodes in obj_list
+        if len(obj_list) > 1 or loop:
+            common_target = ('common_target', 0)
+            self.graph.add_node(common_target)
+            # This is the case when source and target are the same. NetworkX
+            # does not allow loops in the paths, so we work around it by using
+            # target predecessors as new targets
+            if loop:
+                for obj in self.graph.predecessors(list(obj_list)[0]):
+                    self.graph.add_edge(obj, common_target)
+            else:
+                for obj in obj_list:
+                    self.graph.add_edge(obj, common_target)
+            result = self.find_paths(input_set, common_target, max_paths,
+                                     max_path_length, loop, dummy_target=True,
+                                     filter_func=node_filter_func)
+
+            self.graph.remove_node(common_target)
+        else:
+            result = self.find_paths(input_set, list(obj_list)[0], max_paths,
+                                     max_path_length, loop, dummy_target=False,
+                                     filter_func=node_filter_func)
+        if result.path_found:
+            logger.info('Found paths for %s' % stmt)
+            return result
+
+        # If we got here, then there was no path for any observable
+        logger.info('No paths found for %s' % stmt)
+        return self.make_false_result('NO_PATHS_FOUND',
+                                      max_paths, max_path_length)
+
+    def get_all_subjects_objects(self, stmt):
         # Make sure graph is created
         self.get_graph()
         # Extract subject and object info from test statement
         subj_list, obj_list, result_code = self.process_statement(stmt)
         if result_code:
-            return self.make_false_result(result_code, max_paths,
-                                          max_path_length)
+            return None, None, result_code
         # This is the case if we are checking a Statement whose
         # subject is genuinely None
         if all(s is None for s in subj_list):
@@ -241,49 +308,18 @@ class ModelChecker(object):
                     continue
                 input_set += inp
             if not input_set and meaningful_res_code:
-                return self.make_false_result(meaningful_res_code, max_paths,
-                                              max_path_length)
+                return None, None, meaningful_res_code
 
         logger.info('Input set: %s' % str(input_set))
 
-        # If source and target are the same, we need to handle a loop
-        loop = False
-        if (input_set and (len(input_set) == len(obj_list) == 1) and
-                (list(input_set)[0] == list(obj_list)[0])):
-            loop = True
-        # Now we add a dummy target node as a child to all nodes in obj_list
-        common_target = ('common_target', 0)
-        self.graph.add_node(common_target)
-        # This is the case when source and target are the same. NetworkX does
-        # not allow loops in the paths, so we work around it by using target
-        # predecessors as new targets
-        if loop:
-            for obj in self.graph.predecessors(list(obj_list)[0]):
-                self.graph.add_edge(obj, common_target)
-        else:
-            for obj in obj_list:
-                self.graph.add_edge(obj, common_target)
-        result = self.find_paths(input_set, common_target, max_paths,
-                                 max_path_length, loop)
+        # Statement object is None
+        if all(o is None for o in obj_list):
+            obj_list = None
 
-        self.graph.remove_node(common_target)
-        # If a path was found, then we return it; otherwise, that means
-        # there was no path for this object, so we have to try the next
-        # one
-        if result.path_found:
-            logger.info('Found paths for %s' % stmt)
-            return result
-        # Return the result if the subject/input rules were not found
-        if result.result_code in [
-                'SUBJECT_NOT_FOUND', 'INPUT_RULES_NOT_FOUND']:
-            return result
-        # If we got here, then there was no path for any observable
-        logger.info('No paths found for %s' % stmt)
-        return self.make_false_result('NO_PATHS_FOUND',
-                                      max_paths, max_path_length)
+        return input_set, obj_list, None
 
     def find_paths(self, input_set, target, max_paths=1, max_path_length=5,
-                   loop=False):
+                   loop=False, dummy_target=False, filter_func=None):
         """Check for a source/target path in the model.
 
         Parameters
@@ -299,6 +335,12 @@ class ModelChecker(object):
             The maximum length of specific paths to return.
         loop : bool
             Whether we are looking for a loop path.
+        dummy_target : False
+            Whether the target is a dummy node.
+        filter_func : function or None
+            A function to constrain the search. A function should take a node
+            as a parameter and return True if the node is allowed to be in a
+            path and False otherwise. If None, then no filtering is done.
 
         Returns
         -------
@@ -320,11 +362,12 @@ class ModelChecker(object):
         path_lengths = []
         path_metrics = []
         sources = []
-        for source, path_length in self._find_sources(target, input_set):
-            # Path already includes an edge from targets to common target, so
-            # we need to subtract one edge. In case of loops, we are
-            # already missing one edge, there's no need to subtract one more.
-            if not loop:
+        for source, path_length in find_sources(self.graph, target, input_set,
+                                                filter_func):
+            # If a dummy target is used, we need to subtract one edge.
+            # In case of loops, we are already missing one edge, there's no
+            # need to subtract one more.
+            if dummy_target and not loop:
                 path_length = path_length - 1
             # There might be a case when sources and targets contain the same
             # nodes (e.g. different agent state in PyBEL networks) that would
@@ -334,7 +377,7 @@ class ModelChecker(object):
                 pm = PathMetric(source, target, path_length)
                 path_metrics.append(pm)
                 path_lengths.append(path_length)
-                # Keep unique sources but use a list (not set) to preserve order
+                # Keep unique sources but use a list, not set to preserve order
                 if source not in sources:
                     sources.append(source)
         # Now, look for paths
@@ -345,7 +388,7 @@ class ModelChecker(object):
             return pr
         elif path_metrics:
             if min(path_lengths) <= max_path_length:
-                if not loop:
+                if dummy_target and not loop:
                     search_path_length = min(path_lengths) + 1
                 else:
                     search_path_length = min(path_lengths)
@@ -357,8 +400,9 @@ class ModelChecker(object):
                 for source in sources:
                     logger.info('Finding paths between %s and %s'
                                 % (str(source), target))
-                    path_iter = get_path_iter(self.graph, source, target,
-                                              search_path_length, loop)
+                    path_iter = get_path_iter(
+                        self.graph, source, target, search_path_length, loop,
+                        dummy_target, filter_func)
                     for path in path_iter:
                         pr.add_path(tuple(path))
                         # Do not get next path if reached max_paths
@@ -379,69 +423,47 @@ class ModelChecker(object):
             return PathResult(False, 'NO_PATHS_FOUND',
                               max_paths, max_path_length)
 
-    def _find_sources(self, target, sources):
-        """Get the set of source nodes with paths to the target.
+    def make_false_result(self, result_code, max_paths, max_path_length):
+        return PathResult(False, result_code, max_paths, max_path_length)
 
-        Given a common target and  a list of sources (or None if test statement
-        subject is None), perform a breadth-first search upstream from the
-        target to determine whether there are any sources that have paths to
-        the target. For efficiency, does not return the full path,
-        but identifies the upstream sources and the length of the path.
+    def update_filter_func(self, agent_filter_func):
+        """Converts a function filtering agents to a function filtering nodes
 
         Parameters
         ----------
-        target : tuple
-            The node (usually common target node) in the graph to start
-            looking upstream for matching sources.
-        sources : list[tuple]
-            Signed nodes corresponding to the subject or upstream influence
-            being checked.
+        agent_filter_func : function
+            A function to constrain the intermediate nodes in the path. A
+            function should take an agent as a parameter and return True if the
+            agent is allowed to be in a path and False otherwise.
 
         Returns
         -------
-        generator of (source, path_length)
-            Yields tuples of source node (tuple of string and sign) and path
-            length (int). If there are no paths to any of the given source
-            nodes, the generator is empty.
+        node_filter_func : function
+            A new filter function applying the logic from agent_filter_func to
+            nodes instead of agents.
         """
-        # First, create a list of visited nodes
-        # Adapted from
-        # networkx.algorithms.traversal.breadth_first_search.bfs_edges
-        visited = set([target])
-        # Generate list of predecessor nodes with a sign updated according to
-        # the sign of the target node
+        if agent_filter_func is None:
+            return None
 
-        # The queue holds tuples of "parents" (in this case downstream nodes)
-        # and their "children" (in this case their upstream influencers)
-        queue = deque([(target, self.graph.predecessors(target), 0)])
-        while queue:
-            parent, children, path_length = queue[0]
-            try:
-                # Get the next child in the list
-                child = next(children)
-                # Is this child one of the source nodes we're looking for? If
-                # so, yield it along with path length.
-                # Also make sure that found source is positive
-                if (sources is None or child in sources) and child[1] == 0:
-                    logger.debug("Found path to %s from %s with length %d"
-                                 % (target, child, path_length+1))
-                    yield (child, path_length+1)
-                # Check this child against the visited list. If we haven't
-                # visited it already (accounting for the path to the node),
-                # then add it to the queue.
-                if child not in visited:
-                    visited.add(child)
-                    queue.append(
-                        (child, self.graph.predecessors(child), path_length + 1))
-            # Once we've finished iterating over the children of the current
-            # node, pop the node off and go to the next one in the queue
-            except StopIteration:
-                queue.popleft()
-        # There was no path; this will produce an empty generator
-        return
+        def node_filter_func(n):
+            # We're using n[0] here because n is a signed node while
+            # nodes_to_agents contains unsigned nodes (equivalent of n[0])
+            ag = self.nodes_to_agents.get(n[0])
+            if ag is None:
+                logger.warning('Could not get agent for node %s' % n[0])
+                # Do not filter the node if we can't map it to agent
+                return True
+            return agent_filter_func(ag)
 
-    def make_false_result(self, result_code, max_paths, max_path_length):
-        return PathResult(False, result_code, max_paths, max_path_length)
+        logger.info('Converted %s to node filter function'
+                    % agent_filter_func.__name__)
+        return node_filter_func
+
+    def get_nodes_to_agents(self, *args, **kwargs):
+        """Return a dictionary mapping nodes of intermediate signed edges graph
+        to INDRA agents.
+        """
+        raise NotImplementedError("Method must be implemented in child class.")
 
     def get_graph(self, **kwargs):
         """Return a graph  with signed nodes to find the path."""
@@ -486,24 +508,6 @@ class ModelChecker(object):
         raise NotImplementedError("Method must be implemented in child class.")
 
 
-def get_path_iter(graph, source, target, path_length, loop):
-    """Return a generator of paths with path_length cutoff from source to target."""
-    path_iter = nx.all_simple_paths(graph, source, target, path_length)
-    try:
-        for p in path_iter:
-            path = deepcopy(p)
-            # Remove common target from a path.
-            path.remove(target)
-            if loop:
-                path.append(path[0])
-            # A path should contain at least one edge
-            if len(path) < 2:
-                continue
-            yield path
-    except nx.NetworkXNoPath:
-        pass
-
-
 def signed_edges_to_signed_nodes(graph, prune_nodes=True,
                                  edge_signs={'pos': 0, 'neg': 1},
                                  copy_edge_data=False):
@@ -577,4 +581,4 @@ def prune_signed_nodes(graph):
         nodes_to_prune = [node for node, in_deg
                           in graph.in_degree()
                           if in_deg == 0 and node[1] == 1]
-    return graph
+    return graph
\ No newline at end of file
