@@ -9,10 +9,10 @@ from copy import deepcopy
 from indra.statements import Agent
 from indra.databases import hgnc_client
 from indra.util import read_unicode_csv
-from indra.preassembler.grounding_mapper.gilda import get_gilda_models, \
-    run_gilda_disambiguation
-from .standardize import standardize_db_refs, standardize_agent_name
-from .adeft import adeft_disambiguators, run_adeft_disambiguation
+from indra.preassembler.grounding_mapper.gilda import get_gilda_models
+from indra.ontology.standardize import standardize_db_refs, \
+    standardize_agent_name
+from .disambiguate import adeft_disambiguators, DisambManager
 
 logger = logging.getLogger(__name__)
 
@@ -59,6 +59,7 @@ class GroundingMapper(object):
         self.misgrounding_map = misgrounding_map if misgrounding_map \
             else default_misgrounding_map
         self.use_adeft = use_adeft
+        self.disamb_manager = DisambManager()
         self.gilda_mode = gilda_mode
         self._gilda_models = None
 
@@ -146,32 +147,44 @@ class GroundingMapper(object):
                 continue
             # If the agent's TEXT is in the ignores list, we return None to
             # then filter out the Statement
-            agent_txt = agent.db_refs.get('TEXT')
-            if agent_txt and agent_txt in self.ignores:
+            agent_txts = {agent.db_refs[t] for t in {'TEXT', 'TEXT_NORM'}
+                          if t in agent.db_refs}
+            if agent_txts and agent_txts & set(self.ignores):
                 return None
 
             # Check if an adeft model exists for agent text
             adeft_success = False
-            if self.use_adeft and agent_txt and agent_txt in \
-                    adeft_disambiguators:
+            if self.use_adeft and agent_txts and agent_txts & \
+                    set(adeft_disambiguators):
                 try:
-                    adeft_success = run_adeft_disambiguation(mapped_stmt,
-                                                             agent, idx)
+                    # Us the longest match for disambiguation
+                    txt_for_adeft = sorted(agent_txts &
+                                           set(adeft_disambiguators),
+                                           key=lambda x: len(x))[-1]
+                    adeft_success = self.disamb_manager.\
+                        run_adeft_disambiguation(mapped_stmt, agent, idx,
+                                                 txt_for_adeft)
                 except Exception as e:
                     logger.error('There was an error during Adeft'
-                                 ' disambiguation of %s.' % agent_txt)
+                                 ' disambiguation of %s.' % str(agent_txts))
                     logger.error(e)
 
             gilda_success = False
+            # Gilda is not used if agent text is in the grounding map
             if not adeft_success and self.gilda_mode and \
-                    agent_txt in self.gilda_models:
+               not agent_txts & set(self.grounding_map) and \
+               agent_txts & set(self.gilda_models):
                 try:
-                    gilda_success = \
+                    # Us the longest match for disambiguation
+                    txt_for_gilda = sorted(agent_txts & set(self.gilda_models),
+                                           key=lambda x: len(x))[-1]
+                    gilda_success = self.disamb_manager.\
                         run_gilda_disambiguation(mapped_stmt, agent, idx,
+                                                 txt_for_gilda,
                                                  mode=self.gilda_mode)
                 except Exception as e:
                     logger.error('There was an error during Gilda'
-                                 ' disambiguation of %s.' % agent_txt)
+                                 ' disambiguation of %s.' % str(agent_txts))
                     logger.error(e)
 
             # If Adeft and Gilda were not used or didn't succeed, we do
@@ -228,27 +241,37 @@ class GroundingMapper(object):
         agent.db_refs = self.standardize_db_refs(agent.db_refs)
         # If there is no TEXT available, we can return immediately since we
         # can't do mapping
-        agent_text = agent.db_refs.get('TEXT')
-        if not agent_text:
+        agent_txts = sorted({agent.db_refs[t] for t in {'TEXT', 'TEXT_NORM'}
+                             # Note that get here will correctly handle both
+                             # a non-existent entry and a None entry which
+                             # sometimes appears
+                             if agent.db_refs.get(t)}, key=lambda x: len(x),
+                            reverse=True)
+        if not agent_txts:
             # We still do the name standardization here
             if do_rename:
                 self.standardize_agent_name(agent, standardize_refs=False)
             return agent
 
         # 1. Check if there is a full agent mapping and apply if there is
-        if agent_text in self.agent_map:
-            mapped_to_agent = \
-                Agent._from_json(self.agent_map[agent_text]['agent'])
-            return mapped_to_agent
+        for agent_text in agent_txts:
+            if agent_text in self.agent_map:
+                mapped_to_agent = \
+                    Agent._from_json(self.agent_map[agent_text]['agent'])
+                return mapped_to_agent
 
         # 2. Look agent text up in the grounding map
-        if agent_text in self.grounding_map:
-            self.update_agent_db_refs(agent, self.grounding_map[agent_text],
-                                      do_rename)
+        for agent_text in agent_txts:
+            if agent_text in self.grounding_map:
+                self.update_agent_db_refs(agent, self.grounding_map[agent_text],
+                                          do_rename)
+                return agent
 
         # 3. Look agent text up in the misgrounding map
-        if agent_text in self.misgrounding_map:
-            self.remove_agent_db_refs(agent, self.misgrounding_map[agent_text])
+        for agent_text in agent_txts:
+            if agent_text in self.misgrounding_map:
+                self.remove_agent_db_refs(agent,
+                                          self.misgrounding_map[agent_text])
         # This happens when there is an Agent text but it is not in the
         # grounding map. We still do the name standardization here.
         if do_rename:
@@ -292,12 +315,14 @@ class GroundingMapper(object):
         # that are to be eliminated, we consider the Agent's db_refs to be
         # invalid and remove them. We then reset the Agent's name to
         # its TEXT value if available.
-        agent_txt = agent.db_refs.get('TEXT')
+        preserve_refs = {k: agent.db_refs[k] for k in {'TEXT', 'TEXT_NORM'}
+                         if k in agent.db_refs}
         if set(standard_refs.items()) & set(agent.db_refs.items()):
-            agent.db_refs = {}
-            if agent_txt:
-                agent.db_refs['TEXT'] = agent_txt
-                agent.name = agent_txt
+            agent.db_refs = preserve_refs
+            if 'TEXT_NORM' in agent.db_refs:
+                agent.name = agent.db_refs['TEXT_NORM']
+            elif 'TEXT' in agent.db_refs:
+                agent.name = agent.db_refs['TEXT']
 
     @staticmethod
     def standardize_db_refs(db_refs):
@@ -454,7 +479,7 @@ def _get_resource_path(*suffixes):
 
 def _load_default_grounding_map():
     default_grounding_map_path = \
-        _get_resource_path('famplex', 'grounding_map.csv')
+        _get_resource_path('grounding', 'grounding_map.csv')
     gmap = load_grounding_map(default_grounding_map_path, hgnc_symbols=True)
     return gmap
 
