@@ -1,9 +1,7 @@
 import os
 import json
 import gzip
-import pickle
 import pandas
-import rdflib
 import logging
 import requests
 from zipfile import ZipFile
@@ -14,15 +12,7 @@ from indra.util import read_unicode_csv, write_unicode_csv
 from indra.databases.obo_client import OboClient
 from indra.databases import chebi_client, pubchem_client
 from indra.databases.lincs_client import load_lincs_csv
-from indra.preassembler.make_entity_hierarchy import \
-    main as make_ent_hierarchy
-from indra.preassembler.make_activity_hierarchy import \
-    main as make_act_hierarchy
-from indra.preassembler.make_modification_hierarchy import \
-    main as make_mod_hierarchy
-from indra.preassembler.make_cellular_component_hierarchy import \
-    main as make_cellular_component_hierarchy
-from indra.preassembler.hierarchy_manager import get_bio_hierarchies
+from . import load_resource_json, get_resource_path
 
 path = os.path.dirname(__file__)
 logging.basicConfig(format='%(levelname)s: indra/%(name)s - %(message)s',
@@ -213,7 +203,15 @@ def update_chebi_references():
     # Save ChEMBL mapping
     fname = os.path.join(path, 'chebi_to_chembl.tsv')
     logger.info('Saving into %s' % fname)
-    df_chembl = df[df['REFERENCE_DB_NAME']=='ChEMBL']
+    df_chembl = df[df['REFERENCE_DB_NAME'] == 'ChEMBL']
+    # Get additional mappings for compounds in tas
+    df_chembl_tas = pandas.read_csv(os.path.join(path, 'chembl_tas.csv'),
+                                    sep=',')[['chebi_id', 'chembl_id']]
+    df_chembl_tas = df_chembl_tas[~df_chembl_tas.chebi_id.isna()]
+    df_chembl_tas['chebi_id'] = df_chembl_tas.chebi_id.\
+        apply(lambda x: str(int(x)))
+    df_chembl_tas.columns = ['COMPOUND_ID', 'REFERENCE_ID']
+    df_chembl = pandas.concat([df_chembl, df_chembl_tas]).drop_duplicates()
     df_chembl.sort_values(['COMPOUND_ID', 'REFERENCE_ID'], ascending=True,
                           inplace=True)
     df_chembl.to_csv(fname, sep='\t', columns=['COMPOUND_ID', 'REFERENCE_ID'],
@@ -279,8 +277,8 @@ def update_chebi_accessions():
     df_cas.sort_values(['ACCESSION_NUMBER', 'COMPOUND_ID'], ascending=True,
                        inplace=True)
     # Here we need to map to primary ChEBI IDs
-    from indra.databases.chebi_client import chebi_to_primary
-    df_cas.COMPOUND_ID.replace(chebi_to_primary, inplace=True)
+    from indra.databases.chebi_client import get_primary_id
+    df_cas.COMPOUND_ID.replace(get_primary_id, inplace=True)
     df_cas.drop_duplicates(subset=['ACCESSION_NUMBER', 'COMPOUND_ID'],
                            inplace=True)
     df_cas.to_csv(fname, sep='\t',
@@ -354,98 +352,6 @@ def update_bel_chebi_map():
                       (chebi_name, chebi_id)).encode('utf-8'))
 
 
-def _get_chebi_obo_terms():
-    def process_term(term):
-        lines = term.split('\n')[1:]
-        term_id = None
-        parents = []
-        secondaries = []
-        name = None
-        for line in lines:
-            k, v = line.split(': ', maxsplit=1)
-            if k == 'id':
-                term_id = v
-            elif k == 'is_a':
-                parents.append(v)
-            elif k == 'relationship':
-                rel, target = v.split(' ')
-                if rel in ('is_conjugate_acid_of', 'has_functional_parent',
-                           'has_parent_hydride', 'has_role'):
-                    parents.append(target)
-            elif k == 'alt_id':
-                secondaries.append(v)
-            elif k == 'name':
-                name = v
-        return term_id, name, secondaries, parents
-    url = 'ftp://ftp.ebi.ac.uk/pub/databases/chebi/ontology/chebi_lite.obo'
-    fname = 'chebi_lite.obo'
-    logger.info('Downloading %s' % url)
-    urlretrieve(url, fname)
-    with open(fname, 'r') as fh:
-        logger.info('Loading %s' % fname)
-        content = fh.read()
-    chunks = content.split('\n\n')
-    terms = [c for c in chunks if c.startswith('[Term]')]
-    logger.info('Found %d terms' % len(terms))
-    term_entries = [process_term(term) for term in terms]
-    return term_entries
-
-
-def update_chebi_entries():
-    term_entries = _get_chebi_obo_terms()
-    # Make the name and secondary table
-    fname = os.path.join(path, 'chebi_entries.tsv')
-    rows = [['CHEBI_ID', 'NAME', 'SECONDARIES']]
-    for term_id, name, secondaries, parents in term_entries:
-        term_id = term_id.replace('CHEBI:', '')
-        rows.append([term_id, name, ','.join(secondaries)])
-    with open(fname, 'wb') as fh:
-        write_unicode_csv(fname, rows, '\t')
-
-
-def make_chebi_hierarchy():
-    term_entries = _get_chebi_obo_terms()
-    terms_with_parents = [(t, p) for t, _, _, p in term_entries if p]
-    g = rdflib.Graph()
-    indra_rn = \
-        rdflib.Namespace('http://sorger.med.harvard.edu/indra/relations/')
-    chebi_ns = rdflib.Namespace('http://identifiers.org/chebi/')
-    isa = indra_rn.term('isa')
-    for child, parents in terms_with_parents:
-        for parent in parents:
-            g.add((chebi_ns.term(child), isa, chebi_ns.term(parent)))
-    return g
-
-
-def update_entity_hierarchy():
-    logger.info('--Updating entity hierarchy----')
-    fname = os.path.join(path, 'famplex/relations.csv')
-    g = make_ent_hierarchy(fname)
-    g_chebi = make_chebi_hierarchy()
-    g += g_chebi
-    gb = g.serialize(format='nt')
-    gb = gb.replace(b'\n\n', b'\n').strip()
-    rows = b'\n'.join(sorted(gb.split(b'\n')))
-    fname = os.path.join(path, 'entity_hierarchy.rdf')
-    with open(fname, 'wb') as fh:
-        fh.write(rows)
-
-
-def update_modification_hierarchy():
-    logger.info('--Updating modification hierarchy----')
-    make_mod_hierarchy()
-
-
-def update_activity_hierarchy():
-    logger.info('--Updating activity hierarchy----')
-    make_act_hierarchy()
-
-
-def update_cellular_component_hierarchy():
-    logger.info('--Updating cellular component hierarchy----')
-    make_cellular_component_hierarchy()
-
-
 def update_famplex_map():
     logger.info('--Updating FamPlex map----')
     # Currently this is a trivial "copy" of the FamPlex equivalences.csv
@@ -468,8 +374,7 @@ def update_ncit_map():
     url_chebi = 'https://ncit.nci.nih.gov/ncitbrowser/ajax?action=' + \
                 'export_mapping&dictionary=NCIt_to_ChEBI_Mapping&version=1.0'
 
-    url_swissprot = 'https://ncit.nci.nih.gov/ncitbrowser/ajax?action=' \
-                    'export_mapping&uri=https://evs.nci.nih.gov/ftp1/' \
+    url_swissprot = 'https://evs.nci.nih.gov/ftp1/' \
                     'NCI_Thesaurus/Mappings/NCIt-SwissProt_Mapping.txt'
 
     def get_ncit_df(url):
@@ -502,15 +407,21 @@ def update_ncit_map():
 
     # Add UniProt mappings
     ncit_swissprot_file = 'NCIt-SwissProt.txt'
+    # These entries represent non-Uniprot (nuccode?) IDs and
+    # we exclude them for validity
+    exclude_list = {'M74558', 'U37690', 'U65002'}
     save_from_http(url_swissprot, ncit_swissprot_file)
     df_uniprot = pandas.read_csv(ncit_swissprot_file, sep='\t',
                                  index_col=None)
     up_entries = {'Source Code': [], 'Target Coding Scheme': [],
                   'Target Code': []}
     for entry in df_uniprot.iterrows():
+        up_id = entry[1]['SwissProt ID'].strip()
+        if up_id in exclude_list:
+            continue
         up_entries['Source Code'].append(entry[1]['NCIt Code'].strip())
         up_entries['Target Coding Scheme'].append('UP')
-        up_entries['Target Code'].append(entry[1]['SwissProt ID'].strip())
+        up_entries['Target Code'].append(up_id)
     df_uniprot = pandas.DataFrame.from_dict(up_entries)
     df_uniprot.sort_values(['Source Code', 'Target Code'], ascending=True,
                            inplace=True)
@@ -532,14 +443,21 @@ def update_famplex():
                  'grounding_map', 'relations']
     for csv_name in csv_names:
         url = famplex_url_pattern % csv_name
-        save_from_http(url, os.path.join(path,'famplex/%s.csv' % csv_name))
+        save_from_http(url, os.path.join(path, 'famplex',
+                                         '%s.csv' % csv_name))
 
 
-def update_hierarchy_pickle():
-    fname = os.path.join(path, 'bio_hierarchies.pkl')
-    hierarchies = get_bio_hierarchies(from_pickle=False)
-    with open(fname, 'wb') as fh:
-        pickle.dump(hierarchies, fh, protocol=4)
+def update_grounding_map():
+    famplex_gmap = os.path.join(path, 'famplex', 'grounding_map.csv')
+    covid_gmap = os.path.join(path, 'grounding', 'covid_grounding.csv')
+    famplex_rows = list(read_unicode_csv(famplex_gmap))
+    row_len = len(famplex_rows[0])
+    covid_rows = list(read_unicode_csv(covid_gmap))
+    covid_rows = [r + [''] * (row_len - len(r))
+                  for r in covid_rows]
+    all_rows = famplex_rows + covid_rows
+    grounding_map = os.path.join(path, 'grounding', 'grounding_map.csv')
+    write_unicode_csv(grounding_map, all_rows)
 
 
 def update_lincs_small_molecules():
@@ -576,8 +494,9 @@ def update_lincs_proteins():
 
 def update_mesh_names():
     """Update Mesh ID to name and tree number mappings."""
-    url = 'ftp://nlmpubs.nlm.nih.gov/online/mesh/2019/xmlmesh/desc2019.gz'
-    desc_path = os.path.join(path, 'mesh_desc2019.gz')
+    url = ('ftp://nlmpubs.nlm.nih.gov/online/mesh/MESH_FILES/'
+           'xmlmesh/desc2021.gz')
+    desc_path = os.path.join(path, 'mesh_desc2021.gz')
     if not os.path.exists(desc_path):
         logging.info('Download MeSH descriptors from %s', url)
         urlretrieve(url, desc_path)
@@ -602,8 +521,9 @@ def update_mesh_names():
 
 def update_mesh_supplementary_names():
     """Update MeSH ID to name mappings for supplementary terms."""
-    supp_url = 'ftp://nlmpubs.nlm.nih.gov/online/mesh/2019/xmlmesh/supp2019.gz'
-    supp_path = os.path.join(path, 'mesh_supp2019.gz')
+    supp_url = ('ftp://nlmpubs.nlm.nih.gov/online/mesh/MESH_FILES/'
+                'xmlmesh/supp2021.gz')
+    supp_path = os.path.join(path, 'mesh_supp2021.gz')
     if not os.path.exists(supp_path):
         logging.info('Download MeSH supplement from %s', supp_url)
         urlretrieve(supp_url, supp_path)
@@ -622,41 +542,6 @@ def update_mesh_supplementary_names():
     write_unicode_csv(fname, supp_rows, delimiter='\t')
 
 
-def update_mesh_mappings():
-    """Update MeSH mappings to other databases."""
-    from indra.databases import mesh_client
-    url = ('https://raw.githubusercontent.com/indralab/gilda/master/gilda/'
-           'resources/mesh_mappings.tsv')
-    df = pandas.read_csv(url, delimiter='\t', dtype=str, header=None)
-    namespaces = ['efo', 'hp', 'doid']
-    xref_mappings = []
-    for ns in namespaces:
-
-        filename = os.path.join(path, '%s.json' % ns)
-        with open(filename) as file:
-            entries = json.load(file)
-        for entry in entries:
-            db, db_id, name = ns.upper(), entry['id'], entry['name']
-            if not df[(df[3] == ns.upper()) & (df[4] == db_id)].empty:
-                continue
-            # We first need to decide if we prioritize another name space
-            xref_dict = {xr['namespace']: xr['id'] for xr in entry['xrefs']}
-            if 'MESH' in xref_dict or 'MSH' in xref_dict:
-                mesh_id = xref_dict.get('MESH') or xref_dict.get('MSH')
-                if not mesh_id.startswith('D'):
-                    continue
-                mesh_name = mesh_client.get_mesh_name(mesh_id)
-                if not mesh_name:
-                    continue
-                xref_mappings.append(('MESH', mesh_id, mesh_name, db,
-                                      db_id, name))
-    df_extend = pandas.DataFrame(xref_mappings, columns=None, dtype=str)
-    df = df.append(df_extend)
-    df.sort_values(1, inplace=True)  # sort by MeSH ID
-    fname = os.path.join(path, 'mesh_mappings.tsv')
-    df.to_csv(fname, sep='\t', index=None, header=None)
-
-
 def _get_term_name_str(record, name):
     # We then need to look for additional terms related to the
     # preferred concept to get additional names
@@ -738,31 +623,157 @@ def update_go():
     OboClient.update_resource(path, url, 'go', remove_prefix=False)
 
 
+def update_chebi_obo():
+    """Update disease ontology."""
+    url = 'ftp://ftp.ebi.ac.uk/pub/databases/chebi/ontology/chebi_lite.obo'
+    OboClient.update_resource(path, url, 'chebi', remove_prefix=False)
+
+
+def update_drugbank_mappings():
+    """Update mappings from DrugBank to CHEBI/CHEMBL"""
+    # Note that for this to work, PyOBO (https://github.com/pyobo/pyobo) has
+    # to be installed and the DrugBank download
+    # (https://www.drugbank.ca/releases/latest) put into ~/.obo/drugbank/
+    # Note that the DrugBank download requires signing up for an account and
+    # waiting for approval.
+    import pyobo
+    drugbank_chembl = pyobo.get_filtered_xrefs('drugbank', 'chembl.compound')
+    drugbank_chebi = pyobo.get_filtered_xrefs('drugbank', 'chebi')
+    chebi_drugbank = pyobo.get_filtered_xrefs('chebi', 'drugbank')
+    drugbank_names = pyobo.get_id_name_mapping('drugbank')
+    rows = []
+    for drugbank_id, chembl_id in drugbank_chembl.items():
+        rows.append([drugbank_id, 'CHEMBL', chembl_id, 'drugbank'])
+    for drugbank_id, chebi_id in drugbank_chebi.items():
+        rows.append([drugbank_id, 'CHEBI', chebi_id, 'drugbank'])
+    for chebi_id, drugbank_id in chebi_drugbank.items():
+        rows.append([drugbank_id, 'CHEBI', chebi_id, 'chebi'])
+    for drugbank_id, name in drugbank_names.items():
+        rows.append([drugbank_id, 'NAME', name, 'drugbank'])
+    fname = os.path.join(path, 'drugbank_mappings.tsv')
+    header = ['DRUGBANK_ID', 'NAMESPACE', 'ID', 'SOURCE']
+    rows = [header] + sorted(rows)
+    write_unicode_csv(fname, rows, delimiter='\t')
+
+
+def update_identifiers_registry():
+    """Update prefixes and patterns for identifiers namespaces."""
+    url = \
+        'https://registry.api.identifiers.org/resolutionApi/getResolverDataset'
+    res = requests.get(url)
+    regj = res.json()
+    patterns = {entry['prefix']:
+                    {'pattern': entry['pattern'],
+                     'namespace_embedded': entry['namespaceEmbeddedInLui']}
+                for entry in sorted(regj['payload']['namespaces'],
+                                    key=lambda x: x['prefix'])}
+    with open(os.path.join(path, 'identifiers_patterns.json'), 'w') as fh:
+        json.dump(patterns, fh, indent=1)
+
+
+def update_biomappings():
+    """Update mappings from the BioMappings project."""
+    from indra.databases import mesh_client
+    from indra.databases.identifiers import get_ns_id_from_identifiers
+    from biomappings.resources import load_mappings, load_predictions
+
+    # We now construct a mapping dict of these mappings
+    biomappings = defaultdict(list)
+    mappings = load_mappings()
+    predictions = load_predictions()
+    for mappings, mapping_type in ((mappings, 'curated'),
+                                  (predictions, 'predicted')):
+        for mapping in mappings:
+            # We skip anything that isn't an exact match
+            if mapping['relation'] != 'skos:exactMatch':
+                continue
+            # We only accept curated mappings for NCIT
+            if mapping_type == 'predicted' and \
+                    (mapping['source prefix'] == 'ncit' or
+                     mapping['target prefix'] == 'ncit'):
+                continue
+            source_ns, source_id = \
+                get_ns_id_from_identifiers(mapping['source prefix'],
+                                           mapping['source identifier'])
+            target_ns, target_id = \
+                get_ns_id_from_identifiers(mapping['target prefix'],
+                                           mapping['target identifier'])
+            # We only take real xrefs, not refs within a given ontology
+            if source_ns == target_ns:
+                continue
+            biomappings[(source_ns, source_id, mapping['source name'])].append(
+                (target_ns, target_id, mapping['target name']))
+            biomappings[(target_ns, target_id, mapping['target name'])].append(
+                (source_ns, source_id, mapping['target name']))
+
+    mesh_mappings = {k: v for k, v in biomappings.items()
+                     if k[0] == 'MESH'}
+    non_mesh_mappings = {k: [vv for vv in v if vv[0] != 'MESH']
+                         for k, v in biomappings.items()
+                         if k[0] != 'MESH' and k[1] != 'MESH'}
+    rows = []
+    for k, v in non_mesh_mappings.items():
+        for vv in v:
+            rows.append(list(k + vv))
+    rows = sorted(rows, key=lambda x: x[1])
+    write_unicode_csv(get_resource_path('biomappings.tsv'), rows,
+                      delimiter='\t')
+
+    # We next look at mappings to MeSH from EFO/HP/DOID
+    for ns in ['efo', 'hp', 'doid']:
+        for entry in load_resource_json('%s.json' % ns):
+            db, db_id, name = ns.upper(), entry['id'], entry['name']
+            if (db, db_id) in biomappings:
+                continue
+            # We first need to decide if we prioritize another name space
+            xref_dict = {xr['namespace']: xr['id']
+                         for xr in entry.get('xrefs', [])}
+            if 'MESH' in xref_dict or 'MSH' in xref_dict:
+                mesh_id = xref_dict.get('MESH') or xref_dict.get('MSH')
+                if not mesh_id.startswith('D'):
+                    continue
+                mesh_name = mesh_client.get_mesh_name(mesh_id)
+                if not mesh_name:
+                    continue
+                key = ('MESH', mesh_id, mesh_name)
+                if key not in mesh_mappings:
+                    mesh_mappings[key] = [(db, db_id, entry['name'])]
+                else:
+                    mesh_mappings[key].append((db, db_id, entry['name']))
+
+    rows = []
+    for k, v in mesh_mappings.items():
+        for vv in v:
+            rows.append(list(k + vv))
+    rows = sorted(rows, key=lambda x: (x[1], x[2], x[3]))
+    write_unicode_csv(get_resource_path('mesh_mappings.tsv'), rows,
+                      delimiter='\t')
+
+
 def main():
     update_famplex()
     update_famplex_map()
+    update_grounding_map()
     update_hgnc_entries()
     update_kinases()
     update_uniprot_subcell_loc()
-    update_chebi_entries()
+    update_chebi_obo()
     update_chebi_references()
     update_chebi_accessions()
     update_hmdb_chebi_map()
     update_bel_chebi_map()
-    update_entity_hierarchy()
-    update_modification_hierarchy()
-    update_activity_hierarchy()
-    update_cellular_component_hierarchy()
-    update_hierarchy_pickle()
     update_ncit_map()
     update_lincs_small_molecules()
     update_lincs_proteins()
     update_mesh_names()
     update_mesh_supplementary_names()
+    update_biomappings()
     update_mirbase()
     update_doid()
     update_efo()
     update_hpo()
+    update_drugbank_mappings()
+    update_identifiers_registry()
 
 
 if __name__ == '__main__':
