@@ -2,20 +2,24 @@
 
 """Processor for PyBEL."""
 
+import os
 import re
 import logging
 import pybel.dsl as dsl
 import pybel.constants as pc
+import pybel.language
 from collections import defaultdict
 from pybel.struct import has_protein_modification
 from pybel.canonicalize import edge_to_bel
 from bel_resources import get_bel_resource
 from indra.statements import *
-from indra.sources.bel.rdf_processor import bel_to_indra, chebi_name_id
+from indra.util import read_unicode_csv
 from indra.databases import (
     chebi_client, go_client, hgnc_client, mesh_client,
-    mirbase_client, uniprot_client,
+    mirbase_client, uniprot_client, taxonomy_client,
+    identifiers
 )
+from indra.ontology.standardize import standardize_name_db_refs
 from indra.assemblers.pybel.assembler import _pybel_indra_act_map
 
 __all__ = [
@@ -26,7 +30,7 @@ __all__ = [
 logger = logging.getLogger(__name__)
 
 
-_pybel_indra_pmod_map = {
+_pybel_indra_pmod_map_helper = {
     'Ph': 'phosphorylation',
     'Hy': 'hydroxylation',
     'Sumo': 'sumoylation',
@@ -40,6 +44,11 @@ _pybel_indra_pmod_map = {
     'Myr': 'myristoylation',
     'Me': 'methylation',
 }
+_pybel_indra_pmod_map = {
+    v['xrefs'][0]: _pybel_indra_pmod_map_helper[k]
+    for k, v in pybel.language.pmod_mappings.items()
+    if k in _pybel_indra_pmod_map_helper  # More indra statements could be proposed based on this
+}
 
 #: A mapping from the BEL text location annotation to the INDRA ones at
 #: :py:data:`indra.reach.processor._section_list`
@@ -90,9 +99,9 @@ class PybelProcessor(object):
             for node_ix, node in enumerate((u_data, v_data)):
                 if isinstance(node, dsl.ComplexAbundance):
                     self._get_enum_complex(u_data, v_data, k, d, node_ix)
-            subj_activity = _get_activity_condition(d.get(pc.SUBJECT))
-            obj_activity = _get_activity_condition(d.get(pc.OBJECT))
-            obj_to_loc = _get_translocation_target(d.get(pc.OBJECT))
+            subj_activity = _get_activity_condition(d.get(pc.SOURCE_MODIFIER))
+            obj_activity = _get_activity_condition(d.get(pc.TARGET_MODIFIER))
+            obj_to_loc = _get_translocation_target(d.get(pc.TARGET_MODIFIER))
             # If the object is a translocation, this represents a controlled
             # translocation, which we currently do not represent
             if obj_to_loc:
@@ -191,12 +200,12 @@ class PybelProcessor(object):
         self.statements.append(stmt)
 
     def _get_regulate_amount(self, u_data, v_data, k, edge_data):
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
-        obj_agent = get_agent(v_data, edge_data.get(pc.OBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
+        obj_agent = get_agent(v_data, edge_data.get(pc.TARGET_MODIFIER))
         if subj_agent is None or obj_agent is None:
             self.unhandled.append((u_data, v_data, edge_data))
             return
-        obj_mod = edge_data.get(pc.OBJECT)
+        obj_mod = edge_data.get(pc.TARGET_MODIFIER)
         has_deg = (obj_mod and obj_mod.get(pc.MODIFIER) == pc.DEGRADATION)
         rel = edge_data[pc.RELATION]
         if rel == pc.REGULATES:
@@ -215,10 +224,10 @@ class PybelProcessor(object):
         self.statements.append(stmt)
 
     def _get_modification(self, u_data, v_data, k, edge_data):
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
         mods, muts = _get_mods_and_muts(v_data)
         v_data_no_mods = v_data.get_parent()
-        obj_agent = get_agent(v_data_no_mods, edge_data.get(pc.OBJECT))
+        obj_agent = get_agent(v_data_no_mods, edge_data.get(pc.TARGET_MODIFIER))
         if subj_agent is None or obj_agent is None:
             self.unhandled.append((u_data, v_data, k, edge_data))
             return
@@ -231,8 +240,8 @@ class PybelProcessor(object):
 
     def _get_regulate_activity(self, u_data, v_data, k, edge_data):
         # Subject info
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
-        subj_activity = _get_activity_condition(edge_data.get(pc.SUBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
+        subj_activity = _get_activity_condition(edge_data.get(pc.SOURCE_MODIFIER))
         # Object info
         # Note: Don't pass the object modifier data because we don't want to
         # put an activity on the agent
@@ -242,7 +251,7 @@ class PybelProcessor(object):
             activity_type = 'activity'
         else:
             obj_activity_condition = \
-                            _get_activity_condition(edge_data.get(pc.OBJECT))
+                            _get_activity_condition(edge_data.get(pc.TARGET_MODIFIER))
             activity_type = obj_activity_condition.activity_type
             assert obj_activity_condition.is_active is True
         # Check for valid subject/object
@@ -269,7 +278,7 @@ class PybelProcessor(object):
         self.statements.append(stmt)
 
     def _get_active_form(self, u_data, v_data, k, edge_data):
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
         # Don't pass the object modifier info because we don't want an activity
         # condition applied to the agent
         obj_agent = get_agent(v_data)
@@ -277,7 +286,7 @@ class PybelProcessor(object):
             self.unhandled.append((u_data, v_data, edge_data))
             return
         obj_activity_condition = \
-                        _get_activity_condition(edge_data.get(pc.OBJECT))
+                        _get_activity_condition(edge_data.get(pc.TARGET_MODIFIER))
         activity_type = obj_activity_condition.activity_type
         # If the relation is DECREASES, this means that this agent state
         # is inactivating
@@ -287,7 +296,7 @@ class PybelProcessor(object):
         self.statements.append(stmt)
 
     def _get_gef_gap(self, u_data, v_data, k, edge_data):
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
         obj_agent = get_agent(v_data)
         if subj_agent is None or obj_agent is None:
             self.unhandled.append((u_data, v_data, k, edge_data))
@@ -301,7 +310,7 @@ class PybelProcessor(object):
         self.statements.append(stmt)
 
     def _get_conversion(self, u_data, v_data, k, edge_data):
-        subj_agent = get_agent(u_data, edge_data.get(pc.SUBJECT))
+        subj_agent = get_agent(u_data, edge_data.get(pc.SOURCE_MODIFIER))
         # Get the nodes for the reactants and products
         reactant_agents = [get_agent(r) for r in v_data[pc.REACTANTS]]
         product_agents = [get_agent(p) for p in v_data[pc.PRODUCTS]]
@@ -326,9 +335,9 @@ class PybelProcessor(object):
         ev_citation = edge_data.get(pc.CITATION)
         ev_pmid = None
         if ev_citation:
-            cit_type = ev_citation[pc.CITATION_DB]
-            cit_ref = ev_citation[pc.CITATION_IDENTIFIER]
-            if cit_type == pc.CITATION_TYPES[pc.CITATION_TYPE_PUBMED]:
+            cit_type = ev_citation.namespace
+            cit_ref = ev_citation.identifier
+            if cit_type == pc.CITATION_TYPE_PUBMED:
                 ev_pmid = cit_ref
                 ev_ref = None
             else:
@@ -344,13 +353,7 @@ class PybelProcessor(object):
 
         text_location = annotations.pop('TextLocation', None)
         if text_location:
-            # Handle dictionary text_location like {'Abstract': True}
-            if isinstance(text_location, dict):
-                # FIXME: INDRA's section_type entry is meant to contain
-                # a single section string like "abstract" but in principle
-                # pybel could have a list of entries in the TextLocation dict.
-                # Here we just take the first one.
-                text_location = list(text_location.keys())[0]
+            text_location = text_location[0].identifier
             epistemics['section_type'] = \
                 _pybel_text_location_map.get(text_location)
 
@@ -502,6 +505,8 @@ def get_db_refs_by_name(ns, name, node_data):
     elif ns == 'FPLX':
         db_refs = {'FPLX': name}
     elif ns in ('GO', 'GOBP', 'GOCC'):
+        if name == 'cell proliferation':
+            name = 'cell population proliferation'
         go_id = go_client.get_go_id_from_label(name)
         if not go_id:
             logger.info('Could not find GO ID for %s' % name)
@@ -519,9 +524,15 @@ def get_db_refs_by_name(ns, name, node_data):
     # it's clear what namespace the name belongs to
     # FIXME: Full implementation would look up MGI/RGD identifiers from
     # the names, and obtain corresponding Uniprot IDs
-    elif ns in ('MGI', 'RGD'):
-        db_refs = {ns: name}
-    # Map Selventa families to FamPlexes
+    elif ns == 'MGI':
+        up_id = mouse_lookup.get(name)
+        if up_id:
+            db_refs = {'UP': up_id}
+    elif ns == 'RGD':
+        up_id = rat_lookup.get(name)
+        if up_id:
+            db_refs = {'UP': up_id}
+    # Map Selventa families and complexes to FamPlex
     elif ns == 'SFAM':
         db_refs = {'SFAM': name}
         indra_name = bel_to_indra.get(name)
@@ -531,6 +542,15 @@ def get_db_refs_by_name(ns, name, node_data):
         else:
             db_refs['FPLX'] = indra_name
             name = indra_name
+    elif ns == 'SCOMP':
+        db_refs = {'SCOMP': name}
+        indra_name = bel_to_indra.get(name)
+        if indra_name is None:
+            logger.info('Could not find mapping for BEL/SCOMP complex: '
+                        '%s (%s)' % (name, node_data))
+        else:
+            db_refs['FPLX'] = indra_name
+            name = indra_name
     # Map Entrez genes to HGNC/UP
     elif ns in ('EGID', 'ENTREZ', 'NCBIGENE'):
         hgnc_id = hgnc_client.get_hgnc_from_entrez(name)
@@ -549,7 +569,7 @@ def get_db_refs_by_name(ns, name, node_data):
             if mirbase_id:
                 db_refs['MIRBASE'] = mirbase_id
         else:
-            logger.info('Could not map EGID%s to HGNC.' % name)
+            logger.debug('Could not map EGID%s to HGNC.' % name)
             name = 'E%s' % name
     elif ns == 'MIRBASE':
         mirbase_id = mirbase_client.get_mirbase_id_from_mirbase_name(name)
@@ -572,9 +592,20 @@ def get_db_refs_by_name(ns, name, node_data):
             db_refs = {'CHEBI': chebi_id}
         else:
             logger.info('CHEBI name %s not found in map.' % name)
+    # These appear in the name slot but are actually IDs
+    elif ns == 'CHEBIID':
+        chebi_id = identifiers.ensure_chebi_prefix(name)
+        db_refs = {'CHEBI': chebi_id}
+        name = chebi_client.get_chebi_name_from_id(chebi_id)
     # SDIS, SCHEM: Include the name as the ID for the namespace
     elif ns in ('SDIS', 'SCHEM', 'TEXT'):
         db_refs = {ns: name}
+    elif ns == 'TAX':
+        tid = taxonomy_client.get_taxonomy_id(name)
+        if tid:
+            db_refs = {'TAXONOMY': tid}
+        else:
+            logger.info('Could not get taxonomy ID for %s' % name)
     else:
         logger.info("Unhandled namespace: %s: %s (%s)" % (ns, name,
                                                           node_data))
@@ -601,45 +632,36 @@ def get_db_refs_by_ident(ns, ident, node_data):
         The grounding for the given entity.
 
     """
-    name = node_data.name
-    db_refs = None
-    if ns == 'HGNC':
-        name = hgnc_client.get_hgnc_name(ident)
-        if not name:
-            return None, None
-        db_refs = {'HGNC': ident}
-        up_id = _get_up_id(ident)
-        if up_id:
-            db_refs['UP'] = up_id
-        mirbase_id = mirbase_client.get_mirbase_id_from_hgnc_id(ident)
-        if mirbase_id:
-            db_refs['MIRBASE'] = mirbase_id
-    elif ns == 'UP':
-        db_refs = {'UP': ident}
-        hgnc_id = uniprot_client.get_hgnc_id(ident)
-        if hgnc_id:
-            db_refs['HGNC'] = hgnc_id
-            name = hgnc_client.get_hgnc_name(hgnc_id)
-        else:
-            name = uniprot_client.get_gene_name(ident)
-            if not name:
-                return None, None
-    elif ns == 'MIRBASE':
-        db_refs = {'MIRBASE': ident}
-    elif ns in ('MGI', 'RGD', 'CHEBI', 'HMDB', 'MESH', 'FPLX'):
-        db_refs = {ns: ident}
-        # raise ValueError('Identifiers for MGI and RGD databases are not '
-        #                 'currently handled: %s' % node_data)
-    elif ns == 'PUBCHEM.COMPOUND':
-        db_refs = {'PUBCHEM': ident}
-    elif ns == 'PFAM':
-        db_refs = {'PF': ident}
+    ns_list = ['HGNC', 'UNIPROT', 'UP', 'FPLX', 'GO', 'GOBP', 'GOCC',
+               'MESHPP', 'MESHD', 'MESH', 'MGI', 'RGD', 'SFAM', 'EGID',
+               'ENTREZ', 'NCBIGENE', 'MIRBASE', 'CHEBI', 'ECCODE' 'SDIS',
+               'SCHEM', 'TEXT', 'DOID', 'EFO', 'HP', 'PFAM', 'ECCODE',
+               'HGNC.GENEFAMILY', 'HGNC_GROUP', 'NCBITAXON', 'PUBCHEM']
+    ns_mappings = {'UNIPROT': 'UP',
+                   'GOBP': 'GO',
+                   'GOCC': 'GO',
+                   'MESHPP': 'MESH',
+                   'MESHD': 'MESH',
+                   'ENTREZ': 'EGID',
+                   'NCBIGENE': 'EGID',
+                   'NCBITAXON': 'TAXONOMY',
+                   'HGNC.GENEFAMILY': 'HGNC_GROUP',
+                   'CHEBIID': 'CHEBI'}
+    raw_name = node_data.name
+    if ns in ns_list:
+        mapped_ns = ns_mappings.get(ns, ns)
+        raw_db_refs = {mapped_ns: ident}
+        std_name, std_db_refs = standardize_name_db_refs(raw_db_refs)
+        if std_name is None:
+            std_name = raw_name
+        if std_db_refs is None:
+            std_db_refs = raw_db_refs
     else:
         logger.info("Unhandled namespace %s with name %s and "
-                    "identifier %s (%s)." % (ns, name,
-                                             node_data.identifier,
-                                             node_data))
-    return name, db_refs
+                    "identifier %s (%s)." % (ns, raw_name, ident, node_data))
+        std_name = raw_name
+        std_db_refs = None
+    return std_name, std_db_refs
 
 
 def extract_context(annotations, annot_manager):
@@ -663,14 +685,13 @@ def extract_context(annotations, annot_manager):
     """
     def get_annot(annotations, key):
         """Return a specific annotation given a key."""
-        val = annotations.pop(key, None)
-        if val:
-            val_list = [v for v, tf in val.items() if tf]
+        val_list = annotations.pop(key, None)
+        if val_list:
             if len(val_list) > 1:
                 logger.warning('More than one "%s" in annotations' % key)
             elif not val_list:
                 return None
-            return val_list[0]
+            return val_list[0].identifier
         return None
 
     bc = BioContext()
@@ -694,6 +715,10 @@ def extract_context(annotations, annot_manager):
                 db_ns, db_id = ref.split('_', 1)
             else:
                 db_ns, db_id = ns, ref
+            if db_ns == 'CLO':
+                db_ns = 'CL'
+            if db_ns in {'CL', 'UBERON', 'DOID'}:
+                db_id = identifiers.ensure_prefix(db_ns, db_id)
             setattr(bc, indra_name,
                     RefContext(name=ann, db_refs={db_ns: db_id}))
     # Overwrite blank BioContext
@@ -711,6 +736,9 @@ def _get_up_id(hgnc_id):
     up_id = hgnc_client.get_uniprot_id(hgnc_id)
     if not up_id:
         logger.info("No Uniprot ID for HGNC ID %s" % hgnc_id)
+        return None
+    if ',' in up_id:
+        return None
     return up_id
 
 
@@ -737,7 +765,7 @@ class AnnotationManager(object):
         self.failures[key].add(value)
 
 
-def _get_mods_and_muts(node_data):
+def _get_mods_and_muts(node_data: dsl.CentralDogma):
     """Get all modifications and mutations on the PyBEL node.
 
     Parameters
@@ -774,12 +802,11 @@ def _get_mods_and_muts(node_data):
 
         elif isinstance(var, dsl.ProteinModification):
             var_ns = var.entity.namespace
-            if var_ns == pc.BEL_DEFAULT_NAMESPACE:
-                var_id = var.entity.name
-                mod_type = _pybel_indra_pmod_map.get(var_id)
+            if var_ns == 'go':
+                mod_type = _pybel_indra_pmod_map.get(var.entity)
                 if mod_type is None:
                     logger.info("Unhandled modification type %s (%s)",
-                                var_id, node_data)
+                                var.entity.name, node_data)
                     continue
                 mc = ModCondition(mod_type, var.get(pc.PMOD_CODE),
                                   var.get(pc.PMOD_POSITION))
@@ -806,11 +833,10 @@ def _get_activity_condition(node_modifier_data):
     if not effect:
         return ActivityCondition('activity', True)
 
-    activity_ns = effect[pc.NAMESPACE]
-    if activity_ns == pc.BEL_DEFAULT_NAMESPACE:
-        activity_name = effect[pc.NAME]
-        activity_type = _pybel_indra_act_map.get(activity_name)
-        # If an activity type in Bel/PyBEL that is not implemented in INDRA,
+    activity_ns = effect.namespace
+    if activity_ns == 'go':
+        activity_type = _pybel_indra_act_map.get(effect)
+        # If an activity type in BEL/PyBEL that is not implemented in INDRA,
         # return generic activity
         if activity_type is None:
             return ActivityCondition('activity', True)
@@ -833,12 +859,13 @@ def _get_translocation_target(node_modifier_data):
     to_loc_info = transloc_data.get(pc.TO_LOC)
     if not to_loc_info:
         return None
-    to_loc_ns = to_loc_info.get('namespace')
-    to_loc_name = to_loc_info.get('name')
+    to_loc_ns = to_loc_info.namespace
+    to_loc_id = to_loc_info.identifier
+    to_loc_name = to_loc_info.name
     # Only use GO Cellular Component location names
-    if to_loc_ns not in ('GO', 'GOCC', 'GOCCID') or not to_loc_name:
+    if to_loc_ns not in ('GO', 'GOCC', 'GOCCID') or (not to_loc_name and not to_loc_id):
         return None
-    return go_client.get_valid_location(to_loc_name)
+    return go_client.get_valid_location(to_loc_id or to_loc_name)
 
 
 def _has_unhandled_modifiers(node_modifier_data):
@@ -875,3 +902,44 @@ def _parse_mutation(s):
         return None, None, None
     from_aa, position, to_aa = m.groups()
     return position, from_aa, to_aa
+
+
+def _build_famplex_map():
+    fname = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                         os.pardir, os.pardir, 'resources',
+                         'famplex_map.tsv')
+    bel_to_indra = {}
+    csv_rows = read_unicode_csv(fname, delimiter='\t')
+    for row in csv_rows:
+        namespace = row[0]
+        entry = row[1]
+        indra_name = row[2]
+        if namespace == 'BEL':
+            bel_to_indra[entry] = indra_name
+    return bel_to_indra
+
+
+def _build_chebi_map():
+    fname = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                         os.pardir, os.pardir, 'resources',
+                         'bel_chebi_map.tsv')
+    chebi_name_id = {}
+    csv_rows = read_unicode_csv(fname, delimiter='\t')
+    for row in csv_rows:
+        chebi_name = row[0]
+        chebi_id = row[1]
+        chebi_name_id[chebi_name] = chebi_id
+    return chebi_name_id
+
+
+bel_to_indra = _build_famplex_map()
+chebi_name_id = _build_chebi_map()
+
+
+mouse_lookup = {gene_name: up_id for up_id, gene_name in
+                uniprot_client.um.uniprot_gene_name.items()
+                if uniprot_client.is_mouse(up_id)}
+
+rat_lookup = {gene_name: up_id for up_id, gene_name in
+              uniprot_client.um.uniprot_gene_name.items()
+              if uniprot_client.is_rat(up_id)}
