@@ -33,7 +33,8 @@ default_output_fname = 'reach_output.json'
 
 
 def process_pmc(pmc_id, offline=False, url=None,
-                output_fname=default_output_fname):
+                output_fname=default_output_fname,
+                organism_priority=None):
     """Return a ReachProcessor by processing a paper with a given PMC id.
 
     Uses the PMC client to obtain the full text. If it's not available,
@@ -58,6 +59,11 @@ def process_pmc(pmc_id, offline=False, url=None,
     output_fname : Optional[str]
         The file to output the REACH JSON output to.
         Defaults to reach_output.json in current working directory.
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -82,7 +88,8 @@ def process_pmc(pmc_id, offline=False, url=None,
     # Now process the NXML file with the provided arguments
     logger.info('Processing %s with REACH' % pmc_id)
     rp = process_nxml_file(fname, citation=pmid, offline=offline, url=url,
-                           output_fname=output_fname)
+                           output_fname=output_fname,
+                           organism_priority=organism_priority)
     return rp
 
 
@@ -112,6 +119,11 @@ def process_pubmed_abstract(pubmed_id, offline=False, url=None,
     output_fname : Optional[str]
         The file to output the REACH JSON output to.
         Defaults to reach_output.json in current working directory.
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
     **kwargs : keyword arguments
         All other keyword arguments are passed directly to `process_text`.
 
@@ -138,7 +150,8 @@ def process_pubmed_abstract(pubmed_id, offline=False, url=None,
 
 
 def process_text(text, citation=None, offline=False, url=None,
-                 output_fname=default_output_fname, timeout=None):
+                 output_fname=default_output_fname, timeout=None,
+                 organism_priority=None):
     """Return a ReachProcessor by processing the given text.
 
     Parameters
@@ -164,6 +177,11 @@ def process_text(text, citation=None, offline=False, url=None,
     timeout : Optional[float]
         This only applies when reading online (`offline=False`). Only wait for
         `timeout` seconds for the api to respond.
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -183,11 +201,13 @@ def process_text(text, citation=None, offline=False, url=None,
     if json_str:
         with open(output_fname, 'wb') as fh:
             fh.write(json_str)
-        return process_json_str(json_str.decode('utf-8'), citation)
+        return process_json_str(json_str.decode('utf-8'), citation=citation,
+                                organism_priority=organism_priority)
 
 
 def process_nxml_str(nxml_str, citation=None, offline=False,
-                     url=None, output_fname=default_output_fname):
+                     url=None, output_fname=default_output_fname,
+                     organism_priority=None):
     """Return a ReachProcessor by processing the given NXML string.
 
     NXML is the format used by PubmedCentral for papers in the open
@@ -212,6 +232,11 @@ def process_nxml_str(nxml_str, citation=None, offline=False,
     output_fname : Optional[str]
         The file to output the REACH JSON output to.
         Defaults to reach_output.json in current working directory.
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -241,11 +266,13 @@ def process_nxml_str(nxml_str, citation=None, offline=False,
     if json_str:
         with open(output_fname, 'wb') as fh:
             fh.write(json_str)
-        return process_json_str(json_str.decode('utf-8'), citation)
+        return process_json_str(json_str.decode('utf-8'), citation=citation,
+                                organism_priority=organism_priority)
 
 
 def process_nxml_file(file_name, citation=None, offline=False,
-                      url=None, output_fname=default_output_fname):
+                      url=None, output_fname=default_output_fname,
+                      organism_priority=None):
     """Return a ReachProcessor by processing the given NXML file.
 
     NXML is the format used by PubmedCentral for papers in the open
@@ -270,6 +297,11 @@ def process_nxml_file(file_name, citation=None, offline=False,
     output_fname : Optional[str]
         The file to output the REACH JSON output to.
         Defaults to reach_output.json in current working directory.
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -292,10 +324,11 @@ def process_nxml_file(file_name, citation=None, offline=False,
     if json_str:
         with open(output_fname, 'wb') as fh:
             fh.write(json_str)
-        return process_json_str(json_str.decode('utf-8'), citation)
+        return process_json_str(json_str.decode('utf-8'), citation=citation,
+                                organism_priority=organism_priority)
 
 
-def process_json_file(file_name, citation=None):
+def process_json_file(file_name, citation=None, organism_priority=None):
     """Return a ReachProcessor by processing the given REACH json file.
 
     The output from the REACH parser is in this json format. This function is
@@ -309,6 +342,11 @@ def process_json_file(file_name, citation=None):
     citation : Optional[str]
         A PubMed ID passed to be used in the evidence for the extracted INDRA
         Statements. Default: None
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -319,12 +357,13 @@ def process_json_file(file_name, citation=None):
     try:
         with open(file_name, 'rb') as fh:
             json_str = fh.read().decode('utf-8')
-            return process_json_str(json_str, citation)
+            return process_json_str(json_str, citation=citation,
+                                    organism_priority=organism_priority)
     except IOError:
         logger.error('Could not read file %s.' % file_name)
 
 
-def process_json_str(json_str, citation=None):
+def process_json_str(json_str, citation=None, organism_priority=None):
     """Return a ReachProcessor by processing the given REACH json string.
 
     The output from the REACH parser is in this json format.
@@ -337,6 +376,11 @@ def process_json_str(json_str, citation=None):
     citation : Optional[str]
         A PubMed ID passed to be used in the evidence for the extracted INDRA
         Statements. Default: None
+    organism_priority : Optional[list of str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
 
     Returns
     -------
@@ -355,7 +399,8 @@ def process_json_str(json_str, citation=None):
         logger.error('Could not decode JSON string.')
         logger.exception(e)
         return None
-    rp = ReachProcessor(json_dict, citation)
+    rp = ReachProcessor(json_dict, pmid=citation,
+                        organism_priority=organism_priority)
     rp.get_modifications()
     rp.get_complexes()
     rp.get_activation()
