@@ -2,12 +2,15 @@ import os
 import re
 import logging
 import objectpath
+from collections import defaultdict
 
 from indra.statements import *
 from indra.util import read_unicode_csv
-from indra.databases import go_client
-from indra.preassembler.grounding_mapper.standardize import \
-    standardize_db_refs, standardize_agent_name
+from indra.databases import go_client, uniprot_client
+from indra.ontology.standardize import \
+    standardize_db_refs, standardize_agent_name, \
+    standardize_name_db_refs
+from indra.statements.validate import validate_text_refs
 from collections import namedtuple
 
 logger = logging.getLogger(__name__)
@@ -36,15 +39,25 @@ class ReachProcessor(object):
         The PubMed ID associated with the extractions.
     all_events : dict[str, str]
         The frame IDs of all events by type in the REACH extraction.
+    organism_priority : list[str]
+        A list of Taxonomy IDs providing prioritization among organisms
+        when choosing protein grounding. If not given, the default behavior
+        takes the first match produced by Reach, which is prioritized to be
+        a human protein if such a match exists.
     """
-    def __init__(self, json_dict, pmid=None):
+    def __init__(self, json_dict, pmid=None, organism_priority=None):
         self.tree = objectpath.Tree(json_dict)
+        self.organism_priority = organism_priority
         self.statements = []
         self.citation = pmid
         if pmid is None:
             if self.tree is not None:
                 self.citation =\
                     self.tree.execute("$.events.object_meta.doc_id")
+                if not validate_text_refs({'PMID': self.citation}):
+                    logger.debug('The citation added is not a valid '
+                                 'PMID, removing.')
+                    self.citation = None
         self.get_all_events()
 
     def print_event_statistics(self):
@@ -285,12 +298,28 @@ class ReachProcessor(object):
                 continue
             annotations['agents']['coords'] = [controller_coords,
                                                controlled_coords]
-            if r['subtype'] == 'positive-activation':
-                st = Activation(controller_agent, controlled_agent,
-                                evidence=ev)
-            else:
-                st = Inhibition(controller_agent, controlled_agent,
-                                evidence=ev)
+            positive = (r['subtype'] == 'positive-activation')
+            # By default, we choose Activation/Inhibition based on polarity
+            stmt_cls = Activation if positive else Inhibition
+            stmt_kwargs = {}
+            # Here we handle a special case where we have the activation of
+            # a modified form, which we transform into a modification
+            # statement, e.g., Phosphorylation
+            if controlled_agent.mods:
+                # NOTE: can there be more than one mods here?
+                mod = controlled_agent.mods[0]
+                # We check all modification classes here
+                if mod.mod_type in modtype_to_modclass:
+                    stmt_cls = modtype_to_modclass[mod.mod_type]
+                    # We take the residue/position information from
+                    # the modificatio, if available
+                    if mod.residue:
+                        stmt_kwargs['residue'] = mod.residue
+                    if mod.position:
+                        stmt_kwargs['position'] = mod.position
+                controlled_agent.mods = []
+            st = stmt_cls(controller_agent, controlled_agent, **stmt_kwargs,
+                          evidence=ev)
             self.statements.append(st)
 
     def get_translocation(self):
@@ -406,7 +435,7 @@ class ReachProcessor(object):
         # This is the default name, which can be overwritten
         # below for specific database entries
         agent_name = entity_term['text']
-        db_refs = self._get_db_refs(entity_term)
+        db_refs = self._get_db_refs(entity_term, self.organism_priority)
 
         mod_terms = entity_term.get('modifications')
         mods, muts = self._get_mods_and_muts_from_mod_terms(mod_terms)
@@ -419,12 +448,16 @@ class ReachProcessor(object):
         return agent, coords
 
     @staticmethod
-    def _get_db_refs(entity_term):
+    def _get_db_refs(entity_term, organism_priority=None):
         db_refs = {}
         for xr in entity_term['xrefs']:
             ns = xr['namespace']
             if ns == 'uniprot':
-                db_refs['UP'] = xr['id']
+                # Note: we add both full protein and protein chain
+                # IDs here so that we can appli organism prioritization in
+                # a uniform way. Later these will be separated out.
+                up_id = xr['id']
+                db_refs['UP'] = up_id
             elif ns == 'hgnc':
                 db_refs['HGNC'] = xr['id']
             elif ns == 'pfam':
@@ -458,12 +491,42 @@ class ReachProcessor(object):
             # We handle "be" here for compatibility with older versions
             elif ns in ('fplx', 'be'):
                 db_refs['FPLX'] = xr['id']
+            elif ns == 'proonto':
+                db_refs['PR'] = xr['id']
             # These name spaces are ignored
             elif ns in ['uaz']:
                 pass
             else:
                 logger.warning('Unhandled xref namespace: %s' % ns)
         db_refs['TEXT'] = entity_term['text']
+
+        # If we have a UniProt grounding and we have a non-default
+        # organism priority list, we call the prioritization function
+        if db_refs.get('UP'):
+            if organism_priority:
+                # These are all the unique groundings in the alt-xrefs list,
+                # which redundantly lists the same match multiple times because
+                # it enumerates multiple synonyms for organisms redundantly
+                unique_altxrefs = \
+                    set((axr['namespace'], axr['id'])
+                        for axr in entity_term.get('alt-xrefs', []))
+                # This returns a single prioritized UniProt ID or None
+                prioritized_id = \
+                    prioritize_organism_grounding(db_refs['UP'],
+                                                  unique_altxrefs,
+                                                  organism_priority)
+                # If we got an ID, we set the UP grounding to that, otherwise
+                # we keep what we already got from the primary xref
+                if prioritized_id:
+                    db_refs['UP'] = prioritized_id
+            # After all this, we need to separate protein chain grounding
+            # and so if we are dealing with one of those, we pop out the UP
+            # key, split the ID to get the chain ID and add that in the UPPRO
+            # namespace.
+            if '#' in db_refs['UP']:
+                up_id = db_refs.pop('UP', None)
+                db_refs['UPPRO'] = up_id.split('#')[1]
+
         db_refs = standardize_db_refs(db_refs)
         return db_refs
 
@@ -501,7 +564,12 @@ class ReachProcessor(object):
         for mod in mods:
             mod_res, mod_pos = mod
             mod_type_str = mod_term['type'].lower()
+            if mod_type_str == 'unknown':
+                continue
             mod_state = agent_mod_map.get(mod_type_str)
+            # We skip unknown modifications since they are very often
+            # unrelated to PTMs, representing things like "expression"
+            # These are real PTM states
             if mod_state is not None:
                 mc = ModCondition(mod_state[0], residue=mod_res,
                                   position=mod_pos, is_modified=mod_state[1])
@@ -586,7 +654,17 @@ class ReachProcessor(object):
             if not lst:
                 return None
             db_name, db_id = lst[0].split(':', 1)
-            return RefContext(db_refs={db_name.upper(): db_id})
+            db_name = db_name.upper()
+            # Here we are dealing with UniProt subcellular components
+            # so we use a different namespace for those
+            if db_name == 'UNIPROT':
+                db_name = 'UPLOC'
+            # These aren't real groundings
+            elif db_name == 'UAZ':
+                return None
+            standard_name, db_refs = \
+                standardize_name_db_refs({db_name: db_id})
+            return RefContext(standard_name, db_refs=db_refs)
 
         context = BioContext()
         # Example: ['taxonomy:9606']
@@ -732,7 +810,7 @@ class ReachProcessor(object):
         else:
             texts = s.split('/')
 
-        sites = [ReachProcessor._parse_site_text_single(t) for t in texts]
+        sites = [parse_amino_acid_string(t) for t in texts]
 
         # If the first site has a residue, and the remaining sites do not
         # explicitly give a residue (example: Tyr-577/576), then apply the
@@ -747,33 +825,33 @@ class ReachProcessor(object):
 
         return sites
 
-    @staticmethod
-    def _parse_site_text_single(s):
-        s = s.strip()
-        for p in (_site_pattern1, _site_pattern2, _site_pattern3):
-            m = re.match(p, s.upper())
-            if m is not None:
-                residue = get_valid_residue(m.groups()[0])
-                site = m.groups()[1]
-                return Site(residue, site)
-        m = re.match(_site_pattern4, s.upper())
+
+def parse_amino_acid_string(s):
+    s = s.strip()
+    for p in (_site_pattern1, _site_pattern2, _site_pattern3):
+        m = re.match(p, s.upper())
         if m is not None:
-            site = m.groups()[0]
-            residue = m.groups()[1]
+            residue = get_valid_residue(m.groups()[0])
+            site = m.groups()[1]
             return Site(residue, site)
-        for p in (_site_pattern5, _site_pattern6, _site_pattern7):
-            m = re.match(p, s.upper())
-            if m is not None:
-                residue = get_valid_residue(m.groups()[0])
-                site = None
-                return Site(residue, site)
-        m = re.match(_site_pattern8, s.upper())
+    m = re.match(_site_pattern4, s.upper())
+    if m is not None:
+        site = m.groups()[0]
+        residue = m.groups()[1]
+        return Site(residue, site)
+    for p in (_site_pattern5, _site_pattern6, _site_pattern7):
+        m = re.match(p, s.upper())
         if m is not None:
-            site = m.groups()[0]
-            residue = None
+            residue = get_valid_residue(m.groups()[0])
+            site = None
             return Site(residue, site)
-        logger.warning('Could not parse site text %s' % s)
-        return Site(None, None)
+    m = re.match(_site_pattern8, s.upper())
+    if m is not None:
+        site = m.groups()[0]
+        residue = None
+        return Site(residue, site)
+    logger.warning('Could not parse site text %s' % s)
+    return Site(None, None)
 
 
 _site_pattern1 = '([' + ''.join(list(amino_acids.keys())) + '])[-]?([0-9]+)$'
@@ -812,7 +890,6 @@ agent_mod_map = {
     'deribosylation': ('ribosylation', False),
     'methylation': ('methylation', True),
     'demethylation': ('methylation', False),
-    'unknown': ('modification', True),
 }
 
 
@@ -883,3 +960,31 @@ def determine_reach_subtype(event_name):
                 best_match_length = len(ss)
 
     return best_match
+
+
+def prioritize_organism_grounding(first_id, xrefs, organism_priority):
+    """Pick a prioritized organism-specific UniProt ID for a protein."""
+    # We find the organism for the first ID picked by Reach
+    first_organism = uniprot_client.get_organism_id(first_id.split('#')[0])
+    # We take only UniProt groundings from the xrefs list
+    uniprot_ids = [xr[1] for xr in xrefs if xr[0] == 'uniprot']
+    # We group UniProt IDs by their organism ID
+    groundings_by_organism = defaultdict(list)
+    for up_id in uniprot_ids:
+        organism_id = uniprot_client.get_organism_id(up_id.split('#')[0])
+        groundings_by_organism[organism_id].append(up_id)
+    # We then go down the list of prioritized organisms and if we find a match
+    # we return immediately
+    for organism in organism_priority:
+        # If the organism matches the organism of the first xref match given by
+        # Reach then we return that ID
+        if organism == first_organism:
+            return first_id
+        # Otherwiser, if we have a group of matches for the current organism,
+        # we sort the list of matches and return the first one (so that the
+        # result is deterministic, though arbitrary).
+        if organism in groundings_by_organism:
+            return sorted(groundings_by_organism[organism])[0]
+    # If there is no prioritized organism-specific match, we return None
+    # and let the upstream code handle what to do.
+    return None
