@@ -32,6 +32,11 @@ molecule_types = protein_types + \
      'ONT::PHARMACOLOGIC-SUBSTANCE']
 
 
+entity_types = molecule_types + ['ONT::BIOLOGICAL-PROCESS',
+                                 'ONT::CANCER',
+                                 'ONT::MEDICAL-DISORDERS-AND-CONDITIONS']
+
+
 class TripsProcessor(object):
     """The TripsProcessor extracts INDRA Statements from a TRIPS XML.
 
@@ -518,7 +523,7 @@ class TripsProcessor(object):
                 # If it has a type and is not a molecule then we can skip it
                 affected_type = affected.find('type')
                 if affected_type is not None and \
-                    affected_type.text not in molecule_types:
+                    affected_type.text not in entity_types:
                     continue
                 # Otherwise we need to look up the element
                 affected = self.tree.find("TERM/[@id='%s']" % affected_id)
@@ -528,7 +533,7 @@ class TripsProcessor(object):
                 if affected_type is None:
                     continue
                 affected_id = None
-                if affected_type.text not in molecule_types:
+                if affected_type.text not in entity_types:
                     if affected_type.text == 'ONT::QTY':
                         of = affected.find('of')
                         if of is not None:
@@ -954,18 +959,22 @@ class TripsProcessor(object):
             if agent is None:
                 continue
             # Get from location
-            from_loc_tag = event.find("from-location")
-            if from_loc_tag is None:
+            froms = [event.find(from_tag)
+                     for from_tag in ('from', 'from-location')]
+            froms = [f for f in froms if f is not None]
+            if not froms:
                 from_location = None
             else:
-                from_loc_id = from_loc_tag.attrib.get('id')
+                from_loc_id = froms[0].attrib.get('id')
                 from_location = self._get_cell_loc_by_id(from_loc_id)
             # Get to location
-            to_loc_tag = event.find("to-location")
-            if to_loc_tag is None:
+            tos = [event.find(from_tag)
+                   for from_tag in ('to', 'to-location')]
+            tos = [t for t in tos if t is not None]
+            if not tos:
                 to_location = None
             else:
-                to_loc_id = to_loc_tag.attrib.get('id')
+                to_loc_id = tos[0].attrib.get('id')
                 to_location = self._get_cell_loc_by_id(to_loc_id)
             if from_location is None and to_location is None:
                 continue
@@ -1259,9 +1268,13 @@ class TripsProcessor(object):
             # If agent_name is still None take the name of the term as
             # agent name
             if agent_name is None:
-                name = term.find("name")
-                if name is not None and name.text is not None:
-                    agent_name = sanitize_trips_name(name.text)
+                input_txt = _get_input_txt(term)
+                if input_txt:
+                    agent_name = input_txt
+                else:
+                    name = term.find("name")
+                    if name is not None and name.text is not None:
+                        agent_name = sanitize_trips_name(name.text)
             # If after all of this, the agent name is still None
             # then we don't extract this term as an agent
             if agent_name is None:
@@ -1666,6 +1679,12 @@ class TripsProcessor(object):
         all_events = self.tree.findall('EVENT')
         active_event_args = set()
         for event in all_events:
+            ev_type = event.find('type').text
+            # This is a special case of an uninteresting event type
+            # that we don't want to consider as a connection for a TERM
+            # of interes.
+            if ev_type == 'ONT::HAVE-PROPERTY':
+                continue
             if event.attrib.get('id') in self._static_events:
                 continue
             args = event.findall('arg') + \
@@ -1746,6 +1765,15 @@ def _is_base_agent_state(agent):
     return False
 
 
+def _get_input_txt(term):
+    drum_terms = term.findall('drum-terms/drum-term')
+    if drum_terms:
+        input_txt = drum_terms[0].attrib.get('input')
+        if input_txt:
+            return input_txt
+    return None
+
+
 def _get_db_refs(term):
     """Extract database references for a TERM."""
     db_refs = {}
@@ -1755,17 +1783,21 @@ def _get_db_refs(term):
     # term but it contains the term in a raw, non-canonicalized form.
     # The <name> tag only contains the name of the entity but it is
     # canonicalized. For instance, MAP2K1 appears as MAP-2-K-1.
-    agent_text_tag = term.find('name')
-    if agent_text_tag is not None:
-        db_refs['TEXT'] = agent_text_tag.text
-        # If we have some drum-terms, the matched-name of the first
-        # drum-term (e.g. "MAP2K1") is a better value for TEXT than
-        # the name of the TERM (e.g. "MAP-2-K-1") so we put that in there
-        drum_terms = term.findall('drum-terms/drum-term')
-        if drum_terms:
-            matched_name = drum_terms[0].attrib.get('matched-name')
-            if matched_name:
-                db_refs['TEXT'] = matched_name
+    input_txt = _get_input_txt(term)
+    if input_txt:
+        db_refs['TEXT'] = input_txt
+    else:
+        agent_text_tag = term.find('name')
+        if agent_text_tag is not None:
+            db_refs['TEXT'] = agent_text_tag.text
+            # If we have some drum-terms, the matched-name of the first
+            # drum-term (e.g. "MAP2K1") is a better value for TEXT than
+            # the name of the TERM (e.g. "MAP-2-K-1") so we put that in there
+            drum_terms = term.findall('drum-terms/drum-term')
+            if drum_terms:
+                matched_name = drum_terms[0].attrib.get('matched-name')
+                if matched_name:
+                    db_refs['TEXT'] = matched_name
 
     # We make a list of scored grounding terms from the DRUM terms
     grounding_terms = _get_grounding_terms(term)
@@ -1807,6 +1839,8 @@ def _get_db_refs(term):
                 # Skip etc UP entries
                 if ref_ns == 'UP' and ref_id == 'etc':
                     continue
+                elif ref_ns == 'UP' and ref_id.startswith('SL'):
+                    ref_ns = 'UPLOC'
                 try:
                     priority = min(priority, ns_priority[ref_ns])
                 except KeyError:
