@@ -3,8 +3,6 @@ from indra.preassembler.grounding_mapper import GroundingMapper
 from indra.preassembler.grounding_mapper.analysis import *
 from indra.preassembler.grounding_mapper.gilda import ground_statements, \
     get_gilda_models, ground_statement
-from indra.preassembler.grounding_mapper.standardize import \
-    standardize_agent_name, standardize_db_refs
 from indra.statements import Agent, Phosphorylation, Complex, Inhibition, \
     Evidence, BoundCondition
 from indra.util import unicode_strs
@@ -20,7 +18,6 @@ def test_simple_mapping():
     mapped_akt = mapped_stmts[0].sub
     assert mapped_akt.db_refs['TEXT'] == 'Akt'
     assert mapped_akt.db_refs['FPLX'] == 'AKT'
-    assert unicode_strs((akt, stmt, gm, mapped_akt))
 
 
 def test_map_standardize_up_hgnc():
@@ -30,7 +27,8 @@ def test_map_standardize_up_hgnc():
     mapped_stmts = gm.map_stmts([stmt])
     assert len(mapped_stmts) == 1
     st = mapped_stmts[0]
-    assert st.enz.db_refs['HGNC'] == st.sub.db_refs['HGNC']
+    assert st.enz.db_refs['HGNC'] == st.sub.db_refs['HGNC'], \
+        (st.enz.db_refs, st.sub.db_refs)
     assert st.enz.db_refs['UP'] == st.sub.db_refs['UP']
 
 
@@ -135,7 +133,7 @@ def test_ignore():
 
 
 def test_renaming():
-    akt_indra = Agent('pkbA', db_refs={'TEXT': 'Akt', 'FPLX': 'AKT family',
+    akt_indra = Agent('pkbA', db_refs={'TEXT': 'Akt', 'FPLX': 'AKT',
                                        'UP': 'P31749'})
     akt_hgnc_from_up = Agent('pkbA', db_refs={'TEXT': 'Akt', 'UP': 'P31749'})
     akt_other = Agent('pkbA', db_refs={'TEXT': 'Akt'})
@@ -147,14 +145,12 @@ def test_renaming():
     renamed_stmts = gm.rename_agents(stmts)
     assert len(renamed_stmts) == 4
     # Should draw on BE first
-    assert renamed_stmts[0].sub.name == 'AKT family'
+    assert renamed_stmts[0].sub.name == 'AKT'
     # Then on the HGNC lookup from Uniprot
     assert renamed_stmts[1].sub.name == 'AKT1', renamed_stmts[1].sub.name
     # Don't fall back on text if there's no grounding
     assert renamed_stmts[2].sub.name == 'pkbA'
     assert renamed_stmts[3].sub.name == 'tat'
-    assert unicode_strs((akt_indra, akt_hgnc_from_up, akt_other,
-                         tat_up_no_hgnc, stmts, gm, renamed_stmts))
 
 
 def test_save_sentences_unicode():
@@ -180,7 +176,6 @@ def test_hgnc_sym_but_not_up():
     assert mapped_erk.db_refs['TEXT'] == 'ERK1'
     assert mapped_erk.db_refs['HGNC'] == '6871'
     assert mapped_erk.db_refs['UP'] == 'P28482'
-    assert unicode_strs((erk, stmt, gm, mapped_stmts, mapped_erk))
 
 
 def test_up_but_not_hgnc():
@@ -195,7 +190,6 @@ def test_up_but_not_hgnc():
     assert mapped_erk.db_refs['TEXT'] == 'ERK1'
     assert mapped_erk.db_refs['HGNC'] == '6871'
     assert mapped_erk.db_refs['UP'] == 'P28482'
-    assert unicode_strs((erk, stmt, gm, mapped_stmts, mapped_erk))
 
 
 def test_hgnc_but_not_up():
@@ -210,7 +204,6 @@ def test_hgnc_but_not_up():
     assert mapped_erk.db_refs['TEXT'] == 'ERK1'
     assert mapped_erk.db_refs['HGNC'] == '6871'
     assert mapped_erk.db_refs['UP'] == 'P28482'
-    assert unicode_strs((erk, stmt, gm, mapped_stmts, mapped_erk))
 
 
 @raises(ValueError)
@@ -245,9 +238,9 @@ def test_up_with_no_gene_name_with_hgnc_sym():
 def test_multiple_mapped_up():
     ag = Agent('xx', db_refs={'HGNC': '377', 'UP': 'O43687'})
     gm.standardize_agent_name(ag, True)
-    assert ag.db_refs['HGNC'] == '377'
-    assert ag.db_refs['UP'] == 'O43687'
-    assert ag.name == 'AKAP7'
+    assert ag.db_refs['HGNC'] == '377', ag.db_refs
+    assert ag.db_refs['UP'] == 'O43687', ag.db_refs
+    assert ag.name == 'AKAP7', ag.name
 
 
 def test_up_and_mismatched_hgnc():
@@ -275,7 +268,6 @@ def test_up_id_with_no_hgnc_id():
     assert mapped_gag.db_refs['TEXT'] == 'Gag'
     assert mapped_gag.db_refs.get('HGNC') is None
     assert mapped_gag.db_refs['UP'] == 'P04585'
-    assert unicode_strs((gag, stmt, gm, mapped_stmts, mapped_gag))
 
 
 def test_up_id_with_no_gene_name():
@@ -291,7 +283,6 @@ def test_up_id_with_no_gene_name():
     assert mapped_ag.db_refs['TEXT'] == 'NoGN'
     assert mapped_ag.db_refs.get('HGNC') is None
     assert mapped_ag.db_refs['UP'] == 'A0K5Q6'
-    assert unicode_strs((no_gn, stmt, gm, mapped_stmts, mapped_ag))
 
 
 def test_in_place_overwrite_of_gm():
@@ -318,8 +309,10 @@ def test_map_entry_hgnc_and_up():
     mapped_stmts = gm.map_stmts([stmt])
     assert len(mapped_stmts) == 1
     ms = mapped_stmts[0]
-    assert ms.sub.db_refs == {'TEXT': 'NF-kappaB p65', 'UP': 'Q04206',
-                              'HGNC': '9955'}
+    assert ms.sub.db_refs == \
+           {'TEXT': 'NF-kappaB p65', 'UP': 'Q04206',
+            'HGNC': '9955', 'MESH': 'D051996',
+            'EGID': '5970'}, ms.sub.db_refs
 
 
 def test_map_agent():
@@ -332,129 +325,6 @@ def test_map_agent():
     assert mapped_ag.db_refs.get('FPLX') == 'ERK'
 
 
-def test_name_standardize_hgnc_up():
-    a1 = Agent('x', db_refs={'HGNC': '9387'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.name == 'PRKAG3'
-    a1 = Agent('x', db_refs={'UP': 'Q9UGI9'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.name == 'PRKAG3'
-    a1 = Agent('x', db_refs={'UP': 'Q8BGM7'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.name == 'Prkag3'
-
-
-def test_name_standardize_chebi():
-    a1 = Agent('x', db_refs={'CHEBI': '15996'})
-    GroundingMapper.standardize_agent_name(a1, False)
-    assert a1.name == 'GTP'
-
-
-def test_name_standardize_go():
-    a1 = Agent('x', db_refs={'GO': 'GO:0006915'})
-    GroundingMapper.standardize_agent_name(a1, False)
-    assert a1.name == 'apoptotic process'
-
-
-def test_name_standardize_mesh():
-    a1 = Agent('x', db_refs={'MESH': 'D008545'})
-    GroundingMapper.standardize_agent_name(a1, False)
-    assert a1.name == 'Melanoma', a1.name
-
-
-def test_name_standardize_mesh_go():
-    a1 = Agent('x', db_refs={'MESH': 'D058750'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.db_refs['GO'] == 'GO:0001837'
-    assert a1.name == 'epithelial to mesenchymal transition', a1.name
-    a1 = Agent('x', db_refs={'GO': 'GO:0001837'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.db_refs['MESH'] == 'D058750'
-    assert a1.name == 'epithelial to mesenchymal transition', a1.name
-
-
-def test_name_standardize_mesh_other_db():
-    a1 = Agent('x', db_refs={'MESH': 'D001194'})
-    GroundingMapper.standardize_agent_name(a1, True)
-    assert a1.db_refs['CHEBI'] == 'CHEBI:46661'
-    assert a1.name == 'asbestos', a1.name
-
-    db_refs = {'MESH': 'D000067777'}
-    db_refs = standardize_db_refs(db_refs)
-    assert db_refs.get('HGNC') == '3313', db_refs
-    assert db_refs.get('UP') == 'Q12926', db_refs
-    a2 = Agent('x', db_refs=db_refs)
-    standardize_agent_name(a2)
-    assert a2.name == 'ELAVL2'
-
-
-def test_standardize_db_refs_efo_hp_doid():
-    refs = standardize_db_refs({'EFO': '0009502'})
-    assert refs.get('MESH') == 'D000007', refs
-    refs = standardize_db_refs({'MESH': 'D000007'})
-    assert refs.get('EFO') == '0009502', refs
-
-    refs = standardize_db_refs({'HP': 'HP:0031801'})
-    assert refs.get('MESH') == 'D064706', refs
-    refs = standardize_db_refs({'MESH': 'D064706'})
-    assert refs.get('HP') == 'HP:0031801', refs
-
-    # Currently there is no one-to-many mapping in the direction towards MeSH
-    # (there used to be) if there is again, we should test it here
-    #refs = standardize_db_refs({'DOID': 'DOID:0060695'})
-    #assert 'MESH' not in refs
-
-    # One-to-many mappings away from MESH
-    refs = standardize_db_refs({'MESH': 'D000071017'})
-    assert 'DOID' not in refs
-
-    refs = standardize_db_refs({'DOID': 'DOID:0060495'})
-    assert refs.get('MESH') == 'D000067208'
-
-    # This is an xrefs-based mapping that isn't in Gilda's resource file
-    refs = standardize_db_refs({'EFO': '0000694'})
-    assert refs.get('MESH') == 'D045169'
-
-
-def test_standardize_name_efo_hp_doid():
-    ag = Agent('x', db_refs={'HP': 'HP:0031801'})
-    standardize_agent_name(ag)
-    # Name based on MESH mapping
-    assert ag.name == 'Vocal Cord Dysfunction'
-
-    ag = Agent('x', db_refs={'HP': 'HP:0000002'})
-    standardize_agent_name(ag)
-    # Name based on HP itself
-    assert ag.name == 'Abnormality of body height'
-
-    ag = Agent('x', db_refs={'DOID': 'DOID:0014667'})
-    standardize_agent_name(ag)
-    # Name based on MESH mapping
-    assert ag.name == 'Metabolic Diseases'
-
-    ag = Agent('x', db_refs={'EFO': '1002050'})
-    standardize_agent_name(ag)
-    # Name based on MESH mapping
-    assert ag.name == 'Nephritis', (ag.name, ag.db_refs)
-
-    ag = Agent('x', db_refs={'EFO': '0000001'})
-    standardize_agent_name(ag)
-    # Name based on EFO itself
-    assert ag.name == 'experimental factor', (ag.name, ag.db_refs)
-
-
-def test_standardize_uppro():
-    ag = Agent('x', db_refs={'UP': 'P01019'})
-    standardize_agent_name(ag)
-    assert ag.name == 'AGT'
-    ag = Agent('x', db_refs={'UPPRO': 'PRO_0000032458'})
-    standardize_agent_name(ag)
-    assert ag.name == 'Angiotensin-2', ag.name
-    ag = Agent('x', db_refs={'UPPRO': 'PRO_0000032458', 'UP': 'P01019'})
-    standardize_agent_name(ag)
-    assert ag.name == 'Angiotensin-2', ag.name
-
-
 @attr('nonpublic')
 def test_adeft_mapping():
     er1 = Agent('ER', db_refs={'TEXT': 'ER'})
@@ -470,9 +340,10 @@ def test_adeft_mapping():
                                                                 pmid2})])
 
     mapped_stmts1 = gm.map_stmts([stmt1])
-    assert mapped_stmts1[0].sub.name == 'ESR1'
-    assert mapped_stmts1[0].sub.db_refs['HGNC'] == '3467'
-    assert mapped_stmts1[0].sub.db_refs['UP'] == 'P03372'
+    assert mapped_stmts1[0].sub.name == 'ESR', \
+        mapped_stmts1[0].sub.name
+    assert mapped_stmts1[0].sub.db_refs['FPLX'] == 'ESR', \
+        mapped_stmts1[0].sub.db_refs
 
     mapped_stmts2 = gm.map_stmts([stmt2])
     assert mapped_stmts2[0].obj.name == 'endoplasmic reticulum', \
@@ -485,21 +356,34 @@ def test_adeft_mapping():
 
 
 def test_adeft_mapping_non_pos():
-    pcs = Agent('PCS', db_refs={'TEXT': 'PCS'})
-    # This is an exact definition of a non-positive label entry so we
-    # expect that it will be applied as a grounding
-    ev = Evidence(text='post concussive symptoms (PCS)')
-    stmt = Phosphorylation(None, pcs, evidence=[ev])
+    er = Agent('ER', db_refs={'TEXT': 'ER'})
+    # This is an exact definition of a pos_label entry so we
+    # expect that it will be applied as a grounding even though the
+    # Adeft model has low precision for this label.
+    ev = Evidence(text='estradiol (ER)')
+    stmt = Phosphorylation(None, er, evidence=[ev])
     mapped_stmt = gm.map_stmts([stmt])[0]
-    assert 'MESH' in mapped_stmt.sub.db_refs, mapped_stmt.evidence
-
+    assert 'CHEBI' in mapped_stmt.sub.db_refs, mapped_stmt.evidence
+    # This one is not an exact definition so we expect the grounding to
+    # be stripped out.
+    ev = Evidence(text='Estradiol is one of the three estrogen hormones'
+                  'naturally produced in the body.')
+    stmt = Phosphorylation(None, er, evidence=[ev])
+    mapped_stmt = gm.map_stmts([stmt])[0]
+    assert 'CHEBI' not in mapped_stmt.sub.db_refs, mapped_stmt.evidence
+    # This is a non-positive label, and we expect it to be stripped out
+    # whether it's an exact definition or not.
     pcs = Agent('PCS', db_refs={'TEXT': 'PCS', 'MESH': 'xxx'})
-    # There a non-positive entry is implied but not exactly, so
-    # the prior grounding will be removed.
-    ev = Evidence(text='post symptoms concussive concussion')
+    ev = Evidence(text='physical component summary (PCS)')
     stmt = Phosphorylation(None, pcs, evidence=[ev])
     mapped_stmt = gm.map_stmts([stmt])[0]
-    assert 'MESH' not in mapped_stmt.sub.db_refs, mapped_stmt.evidence
+    assert 'MESH' not in mapped_stmt.sub.db_refs, \
+        (mapped_stmt.sub.db_refs, mapped_stmt.evidence)
+    ev = Evidence(text='physical component summary')
+    stmt = Phosphorylation(None, pcs, evidence=[ev])
+    mapped_stmt = gm.map_stmts([stmt])[0]
+    assert 'MESH' not in mapped_stmt.sub.db_refs, \
+        (mapped_stmt.sub.db_refs, mapped_stmt.evidence)
 
 
 def test_misgrounding():
@@ -517,7 +401,8 @@ def test_ground_gilda():
         mek = Agent('Mek', db_refs={'TEXT': 'MEK'})
         erk = Agent('Erk1', db_refs={'TEXT': 'Erk1'})
         stmt = Phosphorylation(mek, erk)
-        ground_statements([stmt], mode=mode)
+        grounded_stmts = ground_statements([stmt], mode=mode)
+        stmt = grounded_stmts[0]
         assert stmt.enz.name == 'MEK', stmt.enz
         assert stmt.enz.db_refs['FPLX'] == 'MEK'
         assert stmt.sub.name == 'MAPK3'
@@ -532,12 +417,13 @@ def test_ground_gilda_source():
     stmts = [Phosphorylation(None, Agent('x', db_refs={'TEXT': 'kras'}),
                              evidence=ev)
              for ev in (ev1, ev2, ev3)]
-    ground_statements(stmts, sources=['trips'])
-    assert stmts[0].sub.name == 'x', stmts[0]
-    assert stmts[1].sub.name == 'x'
-    assert stmts[2].sub.name == 'KRAS'
-    ground_statements(stmts, sources=['reach', 'sparser'])
-    assert all(stmt.sub.name == 'KRAS' for stmt in stmts)
+    grounded_stmts = ground_statements(stmts, sources=['trips'])
+    assert grounded_stmts[0].sub.name == 'x', stmts[0]
+    assert grounded_stmts[1].sub.name == 'x'
+    assert grounded_stmts[2].sub.name == 'KRAS'
+    grounded_stmts = ground_statements(stmts, sources=['reach', 'sparser'])
+    assert all(stmt.sub.name == 'KRAS'
+               for stmt in grounded_stmts[:2])
 
 
 def test_gilda_ground_ungrounded():
@@ -551,8 +437,8 @@ def test_gilda_ground_ungrounded():
     assert ag1.name == 'RAS', ag1
     ground_statement(stmts[1], ungrounded_only=True)
     assert ag2.name == 'RAS'
-    ground_statements([stmts[2]], ungrounded_only=True)
-    assert ag3.name == 'RAS'
+    grounded_stmts = ground_statements([stmts[2]], ungrounded_only=True)
+    assert grounded_stmts[0].sub.name == 'RAS'
 
 
 def test_get_gilda_models():
@@ -615,9 +501,60 @@ def test_gilda_disambiguation_local():
     assert annotations['agents']['gilda'][1][0]['term']['db'] == 'HGNC'
 
 
-def test_uppro_fallback():
-    # This UP chain has no name currently so we can test that the fallback
-    # to naming by the UP ID is working
-    ag = Agent('x', db_refs={'UP': 'Q6IE75', 'UPPRO': 'PRO_0000383648'})
-    standardize_agent_name(ag)
-    assert ag.name == 'Bace2'
+def test_grounding_map_gilda_priority():
+    gm.gilda_mode = 'web'
+    fetal_bovine_serum = Agent('FBS', db_refs={'TEXT': 'FBS'})
+    pmid = '28536624'
+    stmt = Phosphorylation(None, fetal_bovine_serum,
+                           evidence=[Evidence(pmid=pmid,
+                                              text_refs={'PMID': pmid})])
+    mapped_stmts = gm.map_stmts([stmt])
+    annotations = mapped_stmts[0].evidence[0].annotations
+    # agents should not be in annotations if gilda is run. Second condition
+    # added as future proofing in case some future change causes this mapping
+    # to add agent annotations in the future.
+    assert 'agents' not in annotations or \
+        'gilda' not in annotations['agents']
+
+
+def test_text_and_norm_text():
+    gm.gilda_mode = 'local'
+
+    # We should filter out ignores in both TEXT and TEXT_NORM
+    ag = Agent('x', db_refs={'TEXT': 'XREF_BIBR', 'TEXT_NORM': 'ERK'})
+    stmt = Phosphorylation(None, ag)
+    res = gm.map_stmts([stmt])
+    assert not res
+    ag = Agent('x', db_refs={'TEXT': 'ERK', 'TEXT_NORM': 'XREF_BIBR'})
+    stmt = Phosphorylation(None, ag)
+    res = gm.map_stmts([stmt])
+    assert not res
+
+    # We should disambiguate based on both TEXT and TEXT_NORM
+    ag = Agent('x', db_refs={'TEXT': 'AA', 'TEXT_NORM': 'XXX'},)
+    stmt = Phosphorylation(None, ag,
+                           evidence=Evidence(text='Arachidonic acid (AA)'))
+    res = gm.map_stmts([stmt])
+    assert res[0].sub.name == 'arachidonic acid', res[0]
+    ag = Agent('x', db_refs={'TEXT': 'XXX', 'TEXT_NORM': 'AA'})
+    stmt = Phosphorylation(None, ag,
+                           evidence=Evidence(text='Arachidonic acid (AA)'))
+    res = gm.map_stmts([stmt])
+    assert res[0].sub.name == 'arachidonic acid', res[0]
+
+    ag = Agent('x', db_refs={'TEXT': 'XXX', 'TEXT_NORM': 'ERK'})
+    stmt = Phosphorylation(None, ag)
+    res = gm.map_stmts([stmt])
+    assert res[0].sub.name == 'ERK', res[0]
+
+    ag = Agent('x', db_refs={'TEXT': 'ERK', 'TEXT_NORM': 'XXX'})
+    stmt = Phosphorylation(None, ag)
+    res = gm.map_stmts([stmt])
+    assert res[0].sub.name == 'ERK', res[0]
+
+
+def test_none_text_corner_case():
+    ag = Agent('x', db_refs={'TEXT': None, 'TEXT_NORM': None})
+    stmt = Phosphorylation(None, ag)
+    res = gm.map_stmts([stmt])
+    assert res[0].sub.name == 'x', res[0]
