@@ -184,15 +184,11 @@ def test_multiple_enzymes():
                                 offline=offline)
         assert rp is not None
         assert len(rp.statements) == 2
-        s = rp.statements[0]
-        if s.enz.name == 'MAP2K1':
-            assert rp.statements[1].enz.name == 'MAP2K2'
-        else:
-            assert rp.statements[1].enz.name == 'MAP2K1'
-        assert (s.sub.name == 'MAPK3')
-        s = rp.statements[1]
-        assert (s.sub.name == 'MAPK3')
-        assert unicode_strs(rp.statements)
+        stmts = sorted(rp.statements, key=lambda x: x.enz.name)
+        assert stmts[0].enz.name == 'MAP2K1', stmts
+        assert stmts[1].enz.name == 'MAP2K2', stmts
+        assert stmts[0].sub.name == 'MAPK3', stmts
+        assert stmts[1].sub.name == 'MAPK3', stmts
 
 
 def test_activate():
@@ -345,24 +341,24 @@ def assert_pmid(stmt):
 
 def test_process_mod_condition1():
     test_cases = [
-        ('MEK1 activates ERK1 that is phosphorylated.',
+        ('phosphorylated MEK1 activates ERK1.',
          'phosphorylation', None, None, True),
-        ('MEK1 activates ERK1 that is phosphorylated on tyrosine.',
-         'phosphorylation', 'Y', None, True),
-        ('MEK1 activates ERK1 that is phosphorylated on Y185.',
-         'phosphorylation', 'Y', '185', True),
+        ('MEK1 that is phosphorylated on serine activates ERK1.',
+         'phosphorylation', 'S', None, True),
+        ('MEK1 that is phosphorylated on S222 activates ERK1.',
+         'phosphorylation', 'S', '222', True),
         ]
     for offline in offline_modes:
         for sentence, mod_type, residue, position, is_modified in test_cases:
             rp = reach.process_text(sentence, offline=offline)
             assert rp is not None
             assert len(rp.statements) == 1
-            mcs = rp.statements[0].obj.mods
-            assert len(mcs) == 1
-            assert mcs[0].mod_type == mod_type
-            assert mcs[0].residue == residue
-            assert mcs[0].position == position
-            assert mcs[0].is_modified == is_modified
+            mcs = rp.statements[0].subj.mods
+            assert len(mcs) == 1, 'No mods for %s' % sentence
+            assert mcs[0].mod_type == mod_type, mcs
+            assert mcs[0].residue == residue, mcs
+            assert mcs[0].position == position, mcs
+            assert mcs[0].is_modified == is_modified, mcs
 
 
 def test_get_db_refs_up_human():
@@ -373,7 +369,8 @@ def test_get_db_refs_up_human():
         }
     db_refs = ReachProcessor._get_db_refs(entity_term)
     assert db_refs == {'UP': 'Q13422', 'HGNC': '13176',
-                       'TEXT': 'Ikaros'}, db_refs
+                       'TEXT': 'Ikaros', 'MESH': 'C497239',
+                       'EGID': '10320'}, db_refs
 
 
 def test_get_db_refs_up_non_human():
@@ -473,3 +470,64 @@ def test_amount_embedded_in_activation():
     assert isinstance(rp.statements[0], IncreaseAmount)
     assert rp.statements[0].subj is not None
     assert rp.statements[0].obj is not None
+
+
+def test_phosphorylation_regulation():
+    here = os.path.dirname(os.path.abspath(__file__))
+    test_file = os.path.join(here, 'reach_reg_phos.json')
+    rp = reach.process_json_file(test_file)
+    assert rp is not None
+    assert len(rp.statements) == 1
+    stmt = rp.statements[0]
+    assert isinstance(stmt, Phosphorylation), stmt
+    assert not stmt.sub.mods
+
+
+def test_organism_prioritization():
+    here = os.path.dirname(os.path.abspath(__file__))
+    test_file = os.path.join(here, 'reach_reg_phos.json')
+
+    def process(organism_priority, expected_up_id):
+        rp = reach.process_json_file(test_file,
+                                     organism_priority=organism_priority)
+        assert rp.statements[0].enz.db_refs['UP'] == expected_up_id, \
+            rp.statements[0].enz.db_refs['UP']
+
+    # These should all default to the Human protein being prioritized
+    process(None, 'P05019')
+    process(['9606'], 'P05019')
+    # Here we prioritize xenopus
+    process(['8355'], 'P16501')
+    # We now use a list of priorities
+    process(['8355', '9606'], 'P16501')
+    process(['9606', '8355'], 'P05019')
+    # Here Arctic fox amdovirus (1513314) is not an available organism
+    # in the list of groundings so we expect prioritization to fall back
+    # correctly to the first match or another organism in the list
+    process(['1513314'], 'P05019')
+    process(['1513314', '9606'], 'P05019')
+    process(['1513314', '8355'], 'P16501')
+
+
+def test_organism_prioritization_uppro():
+    here = os.path.dirname(os.path.abspath(__file__))
+    test_file = os.path.join(here, 'reach_uppro_organisms.json')
+
+    def process(organism_priority, expected_up_id):
+        rp = reach.process_json_file(test_file,
+                                     organism_priority=organism_priority)
+        assert rp.statements[0].subj.db_refs['UPPRO'] == expected_up_id, \
+            rp.statements[0].subj.db_refs['UPPRO']
+
+    # This is the human protein chain
+    process(None, 'PRO_0000006688')
+    process(['9606'], 'PRO_0000006688')
+    process(['9606', '161274'], 'PRO_0000006688')
+    # Let's try the giant fire-bellied toad next
+    process(['161274'], 'PRO_0000003427')
+    process(['161274', '9606'], 'PRO_0000003427')
+    # Now someting non-existent in the groundings to test correct
+    # fallback behavior
+    process(['1513314'], 'PRO_0000006688')
+    process(['1513314', '9606'], 'PRO_0000006688')
+    process(['1513314', '161274'], 'PRO_0000003427')
