@@ -17,7 +17,8 @@ from indra.util import read_unicode_csv
 from indra.pipeline import register_pipeline
 from indra.mechlinker import MechLinker
 from indra.databases import hgnc_client
-from indra.preassembler.hierarchy_manager import hierarchies
+from indra.ontology.bio import bio_ontology
+from indra.ontology.world import world_ontology
 from indra.preassembler import Preassembler, flatten_evidence
 
 
@@ -28,7 +29,8 @@ def _filter(kwargs, arg_list):
     return dict(filter(lambda x: x[0] in arg_list, kwargs.items()))
 
 
-def dump_statements(stmts, fname, protocol=4):
+@register_pipeline
+def dump_statements(stmts_in, fname, protocol=4):
     """Dump a list of statements into a pickle file.
 
     Parameters
@@ -39,9 +41,10 @@ def dump_statements(stmts, fname, protocol=4):
         The pickle protocol to use (use 2 for Python 2 compatibility).
         Default: 4
     """
-    logger.info('Dumping %d statements into %s...' % (len(stmts), fname))
+    logger.info('Dumping %d statements into %s...' % (len(stmts_in), fname))
     with open(fname, 'wb') as fh:
-        pickle.dump(stmts, fh, protocol=protocol)
+        pickle.dump(stmts_in, fh, protocol=protocol)
+    return stmts_in
 
 
 def load_statements(fname, as_dict=False):
@@ -83,7 +86,9 @@ def load_statements(fname, as_dict=False):
 
 
 @register_pipeline
-def map_grounding(stmts_in, **kwargs):
+def map_grounding(stmts_in, do_rename=True, grounding_map=None,
+                  misgrounding_map=None, agent_map=None, ignores=None, use_adeft=True,
+                  gilda_mode=None, grounding_map_policy='replace', **kwargs):
     """Map grounding using the GroundingMapper.
 
     Parameters
@@ -125,21 +130,19 @@ def map_grounding(stmts_in, **kwargs):
         default_agent_map, default_grounding_map, default_ignores, \
         default_misgrounding_map
     logger.info('Mapping grounding on %d statements...' % len(stmts_in))
-    do_rename = kwargs.get('do_rename', True)
-    ignores = kwargs.get('ignores', default_ignores)
-    gm = kwargs.get('grounding_map')
+    ignores = ignores if ignores else default_ignores
+    gm = grounding_map
     if not gm:
         gm = default_grounding_map
-    elif kwargs.get('grounding_map_policy') == 'extend':
+    elif grounding_map_policy == 'extend':
         default_gm = {k: v for (k, v) in default_grounding_map.items()}
         default_gm.update(gm)
         gm = default_gm
-    misgm = kwargs.get('misgrounding_map', default_misgrounding_map)
-    agent_map = kwargs.get('agent_map', default_agent_map)
+    misgm = misgrounding_map if misgrounding_map else default_misgrounding_map
+    agent_map = agent_map if agent_map else default_agent_map
     gm = GroundingMapper(gm, agent_map=agent_map,
                          misgrounding_map=misgm, ignores=ignores,
-                         use_adeft=kwargs.get('use_adeft', True),
-                         gilda_mode=kwargs.get('gilda_mode', None))
+                         use_adeft=use_adeft, gilda_mode=gilda_mode)
     stmts_out = gm.map_stmts(stmts_in, do_rename=do_rename)
     # Patch wrong locations in Translocation statements
     for stmt in stmts_out:
@@ -310,7 +313,8 @@ def merge_deltas(stmts_in):
 
 
 @register_pipeline
-def map_sequence(stmts_in, **kwargs):
+def map_sequence(stmts_in, do_methionine_offset=True,
+                 do_orthology_mapping=True, do_isoform_mapping=True, **kwargs):
     """Map sequences using the SiteMapper.
 
     Parameters
@@ -349,11 +353,11 @@ def map_sequence(stmts_in, **kwargs):
     """
     from indra.preassembler.sitemapper import SiteMapper, default_site_map
     logger.info('Mapping sites on %d statements...' % len(stmts_in))
-    kwarg_list = ['do_methionine_offset', 'do_orthology_mapping',
-                  'do_isoform_mapping']
     sm = SiteMapper(default_site_map,
                     use_cache=kwargs.pop('use_cache', False),
-                    **_filter(kwargs, kwarg_list))
+                    do_methionine_offset=do_methionine_offset,
+                    do_orthology_mapping=do_orthology_mapping,
+                    do_isoform_mapping=do_isoform_mapping)
     valid, mapped = sm.map_sites(stmts_in)
     correctly_mapped_stmts = []
     for ms in mapped:
@@ -370,7 +374,13 @@ def map_sequence(stmts_in, **kwargs):
 
 
 @register_pipeline
-def run_preassembly(stmts_in, **kwargs):
+def run_preassembly(stmts_in, return_toplevel=True, poolsize=None,
+                    size_cutoff=None, belief_scorer=None, ontology=None,
+                    matches_fun=None, refinement_fun=None, refinement_ns=None,
+                    flatten_evidence=False, flatten_evidence_collect_from=None,
+                    normalize_equivalences=False, normalize_opposites=False,
+                    normalize_ns='WM', run_refinement=True, filters=None,
+                    **kwargs):
     """Run preassembly on a list of statements.
 
     Parameters
@@ -395,13 +405,16 @@ def run_preassembly(stmts_in, **kwargs):
         Instance of BeliefScorer class to use in calculating Statement
         probabilities. If None is provided (default), then the default
         scorer is used.
-    hierarchies : Optional[dict]
-        Dict of hierarchy managers to use for preassembly
-    matches_fun : function
+    ontology : Optional[IndraOntology]
+        IndraOntology object to use for preassembly
+    matches_fun : Optional[function]
         A function to override the built-in matches_key function of statements.
-    refinement_fun : function
+    refinement_fun : Optional[function]
         A function to override the built-in refinement_of function of
         statements.
+    refinement_ns : Optional[set]
+        A set of name spaces over which refinements should be constructed.
+        If not provided, all name spaces are considered.
     flatten_evidence : Optional[bool]
         If True, evidences are collected and flattened via supports/supported_by
         links. Default: False
@@ -414,11 +427,18 @@ def run_preassembly(stmts_in, **kwargs):
         If True, equivalent groundings are rewritten to a single standard one.
         Default: False
     normalize_opposites : Optional[bool]
-        If True, groundings that have opposites in the hierarchy are rewritten
+        If True, groundings that have opposites in the ontology are rewritten
         to a single standard one.
     normalize_ns : Optional[str]
         The name space with respect to which equivalences and opposites are
         normalized.
+    filters : Optional[list[function]]
+        A list of function handles that define filter functions on
+        possible statement refinements. Each function takes
+        a stmts_by_hash dict as its input and returns a dict
+        of possible refinements where the keys are statement hashes
+        and the values are sets of statement hashes that the
+        key statement possibly refines.
     save : Optional[str]
         The name of a pickle file to save the results (stmts_out) into.
     save_unique : Optional[str]
@@ -430,37 +450,36 @@ def run_preassembly(stmts_in, **kwargs):
         A list of preassembled top-level statements.
     """
     dump_pkl_unique = kwargs.get('save_unique')
-    belief_scorer = kwargs.get('belief_scorer')
-    matches_fun = kwargs.get('matches_fun')
-    refinement_fun = kwargs.get('refinement_fun')
-    use_hierarchies = kwargs.get('hierarchies', hierarchies)
+    use_ontology = ontology if ontology is not None else bio_ontology
     be = BeliefEngine(scorer=belief_scorer, matches_fun=matches_fun)
-    pa = Preassembler(use_hierarchies, stmts_in, matches_fun=matches_fun,
-                      refinement_fun=refinement_fun)
-    if kwargs.get('normalize_equivalences'):
+    pa = Preassembler(use_ontology, stmts_in, matches_fun=matches_fun,
+                      refinement_fun=refinement_fun,
+                      refinement_ns=refinement_ns)
+    if normalize_equivalences:
         logger.info('Normalizing equals on %d statements' % len(pa.stmts))
-        pa.normalize_equivalences(kwargs.get('normalize_ns'))
-    if kwargs.get('normalize_opposites'):
+        pa.normalize_equivalences(normalize_ns)
+    if normalize_opposites:
         logger.info('Normalizing opposites on %d statements' % len(pa.stmts))
-        pa.normalize_opposites(kwargs.get('normalize_ns'))
+        pa.normalize_opposites(normalize_ns)
 
-    run_preassembly_duplicate(pa, be, save=dump_pkl_unique)
+    dedupl_stmts = run_preassembly_duplicate(pa, be, save=dump_pkl_unique)
+    if not run_refinement:
+        return dedupl_stmts
 
     dump_pkl = kwargs.get('save')
-    return_toplevel = kwargs.get('return_toplevel', True)
-    poolsize = kwargs.get('poolsize', None)
-    size_cutoff = kwargs.get('size_cutoff', 100)
+    size_cutoff = size_cutoff if size_cutoff else 100
+    if not flatten_evidence_collect_from:
+        flatten_evidence_collect_from = 'supported_by'
     options = {'save': dump_pkl, 'return_toplevel': return_toplevel,
                'poolsize': poolsize, 'size_cutoff': size_cutoff,
-               'flatten_evidence': kwargs.get('flatten_evidence', False),
-               'flatten_evidence_collect_from':
-                   kwargs.get('flatten_evidence_collect_from', 'supported_by')
+               'flatten_evidence': flatten_evidence,
+               'flatten_evidence_collect_from': flatten_evidence_collect_from,
+               'filters': filters
                }
     stmts_out = run_preassembly_related(pa, be, **options)
     return stmts_out
 
 
-@register_pipeline
 def run_preassembly_duplicate(preassembler, beliefengine, **kwargs):
     """Run deduplication stage of preassembly on a list of statements.
 
@@ -489,7 +508,6 @@ def run_preassembly_duplicate(preassembler, beliefengine, **kwargs):
     return stmts_out
 
 
-@register_pipeline
 def run_preassembly_related(preassembler, beliefengine, **kwargs):
     """Run related stage of preassembly on a list of statements.
 
@@ -504,11 +522,6 @@ def run_preassembly_related(preassembler, beliefengine, **kwargs):
         If True, only the top-level statements are returned. If False,
         all statements are returned irrespective of level of specificity.
         Default: True
-    poolsize : Optional[int]
-        The number of worker processes to use to parallelize the
-        comparisons performed by the function. If None (default), no
-        parallelization is performed. NOTE: Parallelization is only
-        available on Python 3.4 and above.
     size_cutoff : Optional[int]
         Groups with size_cutoff or more statements are sent to worker
         processes, while smaller groups are compared in the parent process.
@@ -533,11 +546,11 @@ def run_preassembly_related(preassembler, beliefengine, **kwargs):
     logger.info('Combining related on %d statements...' %
                 len(preassembler.unique_stmts))
     return_toplevel = kwargs.get('return_toplevel', True)
-    poolsize = kwargs.get('poolsize', None)
     size_cutoff = kwargs.get('size_cutoff', 100)
+    filters = kwargs.get('filters', None)
     stmts_out = preassembler.combine_related(return_toplevel=False,
-                                             poolsize=poolsize,
-                                             size_cutoff=size_cutoff)
+                                             size_cutoff=size_cutoff,
+                                             filters=filters)
     # Calculate beliefs
     beliefengine.set_hierarchy_probs(stmts_out)
 
@@ -564,16 +577,18 @@ def run_preassembly_related(preassembler, beliefengine, **kwargs):
 
 
 @register_pipeline
-def filter_by_type(stmts_in, stmt_type, **kwargs):
+def filter_by_type(stmts_in, stmt_type, invert=False, **kwargs):
     """Filter to a given statement type.
 
     Parameters
     ----------
     stmts_in : list[indra.statements.Statement]
         A list of statements to filter.
-    stmt_type : indra.statements.Statement
-        The class of the statement type to filter for.
-        Example: indra.statements.Modification
+    stmt_type : str or indra.statements.Statement
+        The class of the statement type to filter for. Alternatively,
+        a string matching the name of the statement class, e.g.,
+        "Activation" can be used.
+        Example: indra.statements.Modification or "Modification"
     invert : Optional[bool]
         If True, the statements that are not of the given type
         are returned. Default: False
@@ -585,7 +600,8 @@ def filter_by_type(stmts_in, stmt_type, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered statements.
     """
-    invert = kwargs.get('invert', False)
+    if isinstance(stmt_type, str):
+        stmt_type = get_statement_by_name(stmt_type)
     logger.info('Filtering %d statements for type %s%s...' %
                 (len(stmts_in), 'not ' if invert else '',
                  stmt_type.__name__))
@@ -672,7 +688,8 @@ def _any_bound_condition_fails_criterion(agent, criterion):
 
 
 @register_pipeline
-def filter_grounded_only(stmts_in, **kwargs):
+def filter_grounded_only(stmts_in, score_threshold=None, remove_bound=False,
+                         **kwargs):
     """Filter to statements that have grounded agents.
 
     Parameters
@@ -694,12 +711,9 @@ def filter_grounded_only(stmts_in, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered statements.
     """
-    remove_bound = kwargs.get('remove_bound', False)
-
     logger.info('Filtering %d statements for grounded agents...' % 
                 len(stmts_in))
     stmts_out = []
-    score_threshold = kwargs.get('score_threshold')
     for st in stmts_in:
         grounded = True
         for agent in st.agent_list():
@@ -753,7 +767,8 @@ def _agent_is_gene(agent, specific_only):
 
 
 @register_pipeline
-def filter_genes_only(stmts_in, **kwargs):
+def filter_genes_only(stmts_in, specific_only=False, remove_bound=False,
+                      **kwargs):
     """Filter to statements containing genes only.
 
     Parameters
@@ -776,9 +791,6 @@ def filter_genes_only(stmts_in, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered statements.
     """
-    remove_bound = 'remove_bound' in kwargs and kwargs['remove_bound']
-
-    specific_only = kwargs.get('specific_only')
     logger.info('Filtering %d statements for ones containing genes only...' % 
                 len(stmts_in))
     stmts_out = []
@@ -853,7 +865,7 @@ def filter_belief(stmts_in, belief_cutoff, **kwargs):
 
 @register_pipeline
 def filter_gene_list(stmts_in, gene_list, policy, allow_families=False,
-                     **kwargs):
+                     remove_bound=False, invert=False, **kwargs):
     """Return statements that contain genes given in a list.
 
     Parameters
@@ -886,9 +898,6 @@ def filter_gene_list(stmts_in, gene_list, policy, allow_families=False,
     stmts_out : list[indra.statements.Statement]
         A list of filtered statements.
     """
-    invert = kwargs.get('invert', False)
-    remove_bound = kwargs.get('remove_bound', False)
-
     if policy not in ('one', 'all'):
         logger.error('Policy %s is invalid, not applying filter.' % policy)
     else:
@@ -905,11 +914,10 @@ def filter_gene_list(stmts_in, gene_list, policy, allow_families=False,
             hgnc_id = hgnc_client.get_hgnc_id(hgnc_name)
             if not hgnc_id:
                 logger.warning('Could not get HGNC ID for %s.' % hgnc_name)
-            gene_uri = hierarchies['entity'].get_uri('HGNC', hgnc_id)
-            parents = hierarchies['entity'].get_parents(gene_uri)
-            for par_uri in parents:
-                ns, id = hierarchies['entity'].ns_id_from_uri(par_uri)
-                filter_list.append(id)
+                continue
+            parents = bio_ontology.get_parents('HGNC', hgnc_id)
+            filter_list += [db_id for db_ns, db_id in parents
+                            if db_ns == 'FPLX']
     stmts_out = []
 
     if remove_bound:
@@ -963,7 +971,7 @@ def filter_gene_list(stmts_in, gene_list, policy, allow_families=False,
 
 
 @register_pipeline
-def filter_concept_names(stmts_in, name_list, policy, **kwargs):
+def filter_concept_names(stmts_in, name_list, policy, invert=False, **kwargs):
     """Return Statements that refer to concepts/agents given as a list of names.
 
     Parameters
@@ -988,8 +996,6 @@ def filter_concept_names(stmts_in, name_list, policy, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered Statements.
     """
-    invert = kwargs.get('invert', False)
-
     if policy not in ('one', 'all'):
         logger.error('Policy %s is invalid, not applying filter.' % policy)
     else:
@@ -1033,7 +1039,8 @@ def filter_concept_names(stmts_in, name_list, policy, **kwargs):
 
 
 @register_pipeline
-def filter_by_db_refs(stmts_in, namespace, values, policy, **kwargs):
+def filter_by_db_refs(stmts_in, namespace, values, policy, invert=False,
+                      match_suffix=False, **kwargs):
     """Filter to Statements whose agents are grounded to a matching entry.
 
     Statements are filtered so that the db_refs entry (of the given namespace)
@@ -1067,9 +1074,6 @@ def filter_by_db_refs(stmts_in, namespace, values, policy, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered Statements.
     """
-    invert = kwargs.get('invert', False)
-    match_suffix = kwargs.get('match_suffix', False)
-
     if policy not in ('one', 'all'):
         logger.error('Policy %s is invalid, not applying filter.' % policy)
         return
@@ -1114,7 +1118,7 @@ def filter_by_db_refs(stmts_in, namespace, values, policy, **kwargs):
 
 
 @register_pipeline
-def filter_human_only(stmts_in, **kwargs):
+def filter_human_only(stmts_in, remove_bound=False, **kwargs):
     """Filter out statements that are grounded, but not to a human gene.
 
     Parameters
@@ -1134,11 +1138,6 @@ def filter_human_only(stmts_in, **kwargs):
         A list of filtered statements.
     """
     from indra.databases import uniprot_client
-    if 'remove_bound' in kwargs and kwargs['remove_bound']:
-        remove_bound = True
-    else:
-        remove_bound = False
-
     dump_pkl = kwargs.get('save')
     logger.info('Filtering %d statements for human genes only...' %
                 len(stmts_in))
@@ -1151,7 +1150,6 @@ def filter_human_only(stmts_in, **kwargs):
         else:
             return True
 
-
     for st in stmts_in:
         human_genes = True
         for agent in st.agent_list():
@@ -1488,7 +1486,6 @@ def filter_inconsequential_acts(stmts_in, whitelist=None, **kwargs):
     return stmts_out
 
 
-@register_pipeline
 def get_unreachable_mods(stmts_in):
     mods_set = {}
     for stmt in stmts_in:
@@ -1699,7 +1696,7 @@ def filter_transcription_factor(stmts_in, **kwargs):
 
 
 @register_pipeline
-def filter_uuid_list(stmts_in, uuids, **kwargs):
+def filter_uuid_list(stmts_in, uuids, invert=True, **kwargs):
     """Filter to Statements corresponding to given UUIDs
 
     Parameters
@@ -1719,7 +1716,6 @@ def filter_uuid_list(stmts_in, uuids, **kwargs):
     stmts_out : list[indra.statements.Statement]
         A list of filtered statements.
     """
-    invert = kwargs.get('invert', False)
     logger.info('Filtering %d statements for %d UUID%s...' %
                 (len(stmts_in), len(uuids), 's' if len(uuids) > 1 else ''))
     stmts_out = []
@@ -1747,7 +1743,7 @@ def filter_by_curation(stmts_in, curations, incorrect_policy='any',
     ----------
     stmts_in : list[indra.statements.Statement]
         A list of statements to filter.
-    curations : list[Curation]
+    curations : list[dict]
         A list of curations for evidences. Curation object should have
         (at least) the following attributes:
         pa_hash (preassembled statement hash), source_hash (evidence hash) and
@@ -1772,18 +1768,19 @@ def filter_by_curation(stmts_in, curations, incorrect_policy='any',
     # curations). Incorrect is a set of hashes of statements that only have
     # incorrect curations (for all or some of the evidences). These sets do
     # not intersect.
-    correct = {c.pa_hash for c in curations if c.tag in correct_tags}
-    incorrect = {c.pa_hash for c in curations if c.pa_hash not in correct}
+    correct = {c['pa_hash'] for c in curations if c['tag'] in correct_tags}
+    incorrect = {c['pa_hash'] for c in curations if c['pa_hash'] not in correct}
     # Store evidence level curations for overall correct statements
     correct_stmt_evid = {}
     for c in curations:
-        if c.pa_hash in correct:
-            if c.pa_hash not in correct_stmt_evid:
-                correct_stmt_evid[c.pa_hash] = defaultdict(set)
-            if c.tag in correct_tags:
-                correct_stmt_evid[c.pa_hash]['correct'].add(c.source_hash)
+        pa_hash = c['pa_hash']
+        if pa_hash in correct:
+            if pa_hash not in correct_stmt_evid:
+                correct_stmt_evid[pa_hash] = defaultdict(set)
+            if c['tag'] in correct_tags:
+                correct_stmt_evid[pa_hash]['correct'].add(c['source_hash'])
             else:
-                correct_stmt_evid[c.pa_hash]['incorrect'].add(c.source_hash)
+                correct_stmt_evid[pa_hash]['incorrect'].add(c['source_hash'])
     stmts_out = []
     logger.info('Filtering %d statements with %s incorrect curations...' %
                 (len(stmts_in), incorrect_policy))
@@ -1797,16 +1794,17 @@ def filter_by_curation(stmts_in, curations, incorrect_policy='any',
 
     def process_and_append(stmt, stmt_list):
         # Filter out incorrect evidences for correct statements
-        if stmt.get_hash() in correct_stmt_evid:
+        stmt_hash = stmt.get_hash()  # Already refreshed when this is called
+        if stmt_hash in correct_stmt_evid:
             evidence = []
             for evid in stmt.evidence:
-                if _is_incorrect(stmt.get_hash(), evid.get_source_hash()):
+                if _is_incorrect(stmt_hash, evid.get_source_hash()):
                     continue
                 else:
                     evidence.append(evid)
             stmt.evidence = evidence
         # Set belief to one for statements with correct curations
-        if update_belief and stmt.get_hash() in correct:
+        if update_belief and stmt_hash in correct:
             stmt.belief = 1
         stmt_list.append(stmt)
 
@@ -1814,7 +1812,8 @@ def filter_by_curation(stmts_in, curations, incorrect_policy='any',
         # Filter statements that have SOME incorrect and NO correct curations
         # (i.e. their hashes are in incorrect set)
         for stmt in stmts_in:
-            if stmt.get_hash() not in incorrect:
+            stmt_hash = stmt.get_hash(refresh=True)
+            if stmt_hash not in incorrect:
                 process_and_append(stmt, stmts_out)
     elif incorrect_policy == 'all':
         # Filter out statements in which ALL evidences are curated
@@ -1822,14 +1821,15 @@ def filter_by_curation(stmts_in, curations, incorrect_policy='any',
         # First, map curated statements to curated evidences.
         incorrect_stmt_evid = defaultdict(set)
         for c in curations:
-            if c.pa_hash in incorrect:
-                incorrect_stmt_evid[c.pa_hash].add(c.source_hash)
+            if c['pa_hash'] in incorrect:
+                incorrect_stmt_evid[c['pa_hash']].add(c['source_hash'])
         for stmt in stmts_in:
             # Compare set of evidence hashes of given statements to set of
             # hashes of curated evidences.
-            if stmt.get_hash() in incorrect_stmt_evid and (
+            stmt_hash = stmt.get_hash(refresh=True)
+            if stmt_hash in incorrect_stmt_evid and (
                     {ev.get_source_hash() for ev in stmt.evidence} <=
-                    incorrect_stmt_evid[stmt.get_hash()]):
+                    incorrect_stmt_evid[stmt_hash]):
                 continue
             else:
                 process_and_append(stmt, stmts_out)
@@ -1855,7 +1855,7 @@ def expand_families(stmts_in, **kwargs):
     """
     from indra.tools.expand_families import Expander
     logger.info('Expanding families on %d statements...' % len(stmts_in))
-    expander = Expander(hierarchies)
+    expander = Expander(bio_ontology)
     stmts_out = expander.expand_families(stmts_in)
     logger.info('%d statements after expanding families...' % len(stmts_out))
     dump_pkl = kwargs.get('save')
@@ -1934,6 +1934,11 @@ def standardize_names_groundings(stmts):
     NOTE: this function is currently optimized for Influence Statements
     obtained from Eidos, Hume, Sofia and CWMS. It will possibly yield
     unexpected results for biology-specific Statements.
+
+    Parameters
+    ----------
+    stmts : list[indra.statements.Statement]
+        A list of statements whose Concept names should be standardized.
     """
     print('Standardize names to groundings')
     for stmt in stmts:
@@ -2004,7 +2009,6 @@ def rename_db_ref(stmts_in, ns_from, ns_to, **kwargs):
     return stmts_out
 
 
-@register_pipeline
 def align_statements(stmts1, stmts2, keyfun=None):
     """Return alignment of two lists of statements by key.
 
@@ -2052,32 +2056,30 @@ def align_statements(stmts1, stmts2, keyfun=None):
     return matches
 
 
-if __name__ == '__main__':
-    if len(sys.argv) < 3:
-        logger.error('Usage: assemble_corpus.py <pickle_file> <output_folder>')
-        sys.exit()
-    stmts_fname = sys.argv[1]
-    out_folder = sys.argv[2]
+@register_pipeline
+def filter_complexes_by_size(stmts_in, members_allowed=5):
+    """Filter out Complexes if the number of members exceeds specified allowed
+    number.
 
-    stmts = load_statements(stmts_fname)
+    Parameters
+    ----------
+    stmts_in : list[indra.statements.Statement]
+        A list of statements from which large Complexes need to be filtered out
+    members_allowed : Optional[int]
+        Allowed number of members to include. Default: 5
 
-    logger.info('All statements: %d' % len(stmts))
-
-    cache_pkl = os.path.join(out_folder, 'mapped_stmts.pkl')
-    options = {'save': cache_pkl, 'do_rename': True}
-    stmts = map_grounding(stmts, **options)
-
-    cache_pkl = os.path.join(out_folder, 'sequence_valid_stmts.pkl')
-    options = {'save': cache_pkl}
-    mapped_stmts = map_sequence(stmts, **options)
-
-    be = BeliefEngine()
-    pa = Preassembler(hierarchies, mapped_stmts)
-
-    cache_pkl = os.path.join(out_folder, 'unique_stmts.pkl')
-    options = {'save': cache_pkl}
-    unique_stmts = run_preassembly_duplicate(pa, be, **options)
-
-    cache_pkl = os.path.join(out_folder, 'top_stmts.pkl')
-    options = {'save': cache_pkl}
-    stmts = run_preassembly_related(pa, be, **options)
+    Returns
+    -------
+    stmts_out : list[indra.statements.Statement]
+        A list of filtered Statements.
+    """
+    stmts_out = []
+    logger.info('Filtering out Complexes with more than %d members from %d '
+                'statements...' % (members_allowed, len(stmts_in)))
+    for stmt in stmts_in:
+        if isinstance(stmt, Complex) and len(stmt.members) > members_allowed:
+            continue
+        else:
+            stmts_out.append(stmt)
+    logger.info('%d statements after filter...' % len(stmts_out))
+    return stmts_out
